/* VNA-lpg  dsPic30f4011 driver program for the N2PK Vector Network Analyzer
 * This version is configured for the W8WWV board which has the
 * LTC2440 fast a/d chip installed.
 *
 *  Copyright (C) 2006 By Lawrence Glaister VE7IT
 *             (ve7it@shaw.ca)
 *
 * I would like to thank Greg Ordy W8WWV for his help and ideas. It was 
 * his contribution of source code for windows ibraries known as VNAccess
 * and VNAgra that made this program possible. 
 * C++ code from VNAgra was converted to the dumb C compiler of the dspic.
 * This code is not as generalized as Gregs, and was
 * stripped to the bare minimum with intent of being being able to 
 * eventually run it on a dsPic30F4011 microcontroller.
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Compile using: mplab ide
 */

// From the VNA Schematic:
//	LO DDS Data, DB25-3, LO DDS serial data in (D1)
//	RF DDS Data, DB25-2, RF DDS serial data in (D0)
//	DDS Reset, DB25-6, Shared DDS Reset in (D4)
//	DDS W_Clk, DB25-4, Shared DDS W_Clk in (D2)
//	DDS FQ_UD, DB25-5, Shared DDS FQ_UD in (D3)
//
//	ADC nCS_1, DB25-8, ADC nCS in (D6)
//	ADC SCK, DB25-7, ADC SCK in (D5)
//	ADC nSDO_1, DB25-11, inversion of ADC SDO out (IOBaseAddress+1, D7 [MSB])
//		Since this is the parallel port BUSY signal, which is inverted in
//		hardware at the computer, the BUSY read by software is the correct value!
//
//	VNA-Plus update: ADC (LTC2440) SDI, DB25-2, (D0)
//
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>		// for PI etc
#include "vna.h"

extern volatile int but1;				// -DEBOUNCE to +DEBOUNCE
extern volatile int but2;
extern volatile int but3;

extern volatile unsigned short int timer_vna;
extern char buff30[30];
//=============================================================================
// Stuff that may need to change for your hardware environment
//=============================================================================

//=============================================================================
// local defines.... you should not have to mess with these
//=============================================================================
int LTC2440_ReadCDS(long int *rdg);
char VNA_GetReadingCDS( long int freq, char br);  // br is used for 'break-in'
double ADCtoVolts(long int rdg);
char GetComplexRdg(long int freq, char br);  // br is used for 'break-in'

//=============================================================================
// local global vars used by this program
//=============================================================================
// because of limited ram and eeprom for saving cal,
// we need to pull some tricks to get it all done
// structure with all calibration constants that is stored in eeprom
// aligned on 32 byte boundary
// seed eeprom with a few default values
// this forces a cal after a new pgm load

extern int ReadEE(int Page, int Offset, int* DataOut, int Size);
extern int EraseEE(int Page, int Offset, int Size);
extern int WriteEE(int* DataIn, int Page, int Offset, int Size);

struct EEbuffer                          // rows after first - also 32 bytes each
{  double Mor,Moi;                     // open
   double Msr,Msi;                     // short
   double Mlr,Mli;                     // load
   double Mtr,Mti;                     // thru
};

struct EEbuffer0                         // 1st row (32 bytes)
{  long int refclk;			 // 156.25 MHz clk
   unsigned long int f_start, f_stop;	 // freq in 0.1 Hz for calibration points
   unsigned long int lastreffreq, lasttrnfreq;
   unsigned long int lastdelf;
   long int cksum;
   char samplerate;			 // a sample rate index: 0,1,2
   char Brdg;                             // 0=Std, 1=Lo-Z, 2=Hi-Z, 3=RFIV
   char lastresultrefnum,lastresulttrnnum;
};

extern char Brdg;    // 0=Std, 1=Lo-Z, 2=Hi-Z, 3=RFIV

struct EEbuffer eeb;      // long int
//struct EEbufferd eebd;    // double

struct EEbuffer0 eeb0;

void readCAL(long int ci)  // ci ranges 0...(CALPTS-1) and is an index to a freq's worth of cal data 
{  int *dptr = &eeb;
   unsigned int page=0x7F, offsetbase=0xFC20, offset, size=16;
   offset=offsetbase + 32*ci;
   ReadEE(page,offset,dptr,size);
   //eeb now loaded with ci's cal data: eeb.Mor, eeb.Moi, eeb.Msr, eeb.Msi, eeb.Mlr, eeb.Mli, eeb.Mtr, eeb.Mti
}

void readCAL0(void)  // reads the first 32 bytes in EEPROM 
{  int *dptr = &eeb0;
   unsigned int page=0x7F, offset=0xFC00, size=16;
   ReadEE(page,offset,dptr,size);
   //eeb0 now loaded with first 32 bytes  of cal data: eeb0.refclk, 
   //eeb0.samplerate , eeb0.fstart, eeb0.fstop , eeb0.cksum, eeb0.Brdg
}

void writeCAL(long int ci)  // ci ranges 0...(CALPTS-1) and is an index to a freq's worth of cal data 
{  int *dptr = &eeb;
   unsigned int page=0x7F, offsetbase=0xFC20, offset, size=16;
   offset=offsetbase + 32*ci;
   EraseEE(page,offset,size);
   WriteEE(dptr,page,offset,size);
}

void writeCAL0(void)  // writes the first 32 bytes in EEPROM 
{  int *dptr = &eeb0;
   unsigned int page=0x7F, offset=0xFC00, size=16;
   EraseEE(page,offset,size);
   WriteEE(dptr,page,offset,size);
}

extern void lcd_goto( int );
extern void lcd_puts( char * );
extern void int_freq1( unsigned long freq );
extern long int calc_cksum(void);

extern __complex__ double Mo,Ms,Ml,Mt,x,y,z;

extern double Rs,Xs,Cs,Ls;				
extern double Rp,Xp,Gp,Bp,Cp,Lp;			
extern double Q;					
extern double SWR,RL;					
extern double magrho,angrho,Rerho,Imrho;
extern double magZu,angZu,magYu,angYu;		
       
       // unknown reading
extern __complex__ double Mu;		        // raw voltages
extern __complex__ double rho;			// computed reflection coeff.
extern __complex__ double Zu,Yu;			// computed impedance
extern __complex__ double Zo;

extern __complex__ double Vrfiv,Irfiv;

long int POSCNTlast=0;
long int ReRdg,ImRdg;
void timerVNAdelay(int delay);

void Vsensor(void)
{    RFIV_SW=1;
}

void Isensor(void)
{    RFIV_SW=0;
}

// an alternate method is to save cal in code space...
// large calibration array in nonvolatile code space
// struct CAL cal[CALPTS] __attribute__ ((space(auto_psv))) ;
// reads are fine from C, but how to write into code space
// has not been figured out yet (needs asm utility)

//=============================================================================
// low level access to a short delay (about 1us)
// On the PC, the isa overhead makes bit flips take 1-2us
// with the dspic, the i/o is too screaming fast
// a loop setting i/o and repeatedly calling this routine
// gives a square wave with about 1.5us high and 1.5us low times.
// not exactly 1us, but close enough for retiming i/o.
//=============================================================================
void dly_1us( void )
{
	static long long int dummy;
	dummy++;
}
//=============================================================================
// low level access to AD9851 local osc data 
// PC - on parallel port pin 3, DO.1  
// dspic RB1
//=============================================================================
void AD9851_LODAT( int value )
{
    if ( value )
		_RB1 = 1;
    else
		_RB1 = 0;
	dly_1us();
}

//=============================================================================
// low level access to AD9851 RF osc data 
// on parallel port pin 2, DO.0  
// dspic RB2
//=============================================================================
void AD9851_RFDAT( int value )
{
    if ( value )
        _RB2 = 1;
    else
        _RB2 = 0;
	dly_1us();
}

//=============================================================================
// low level access to AD9851 reset line on both dds's 
// on parallel port pin 6, DO.4  
// dspic RB3
//=============================================================================
void AD9851_RESET( int value )
{
    if ( value )
        _RB3=1;
    else
        _RB3=0;
	dly_1us();
}

//=============================================================================
// low level access to AD9851 clk line on both dds's 
// on parallel port pin 4, DO.2  
// dspic RB6
//=============================================================================
void AD9851_CLK( int value )
{
    if ( value )
        _RB6 = 1;
    else
        _RB6 = 0;
	dly_1us();
}

//=============================================================================
// low level access to AD9851 FQ_UD line on both dds's 
// on parallel port pin 5, DO.3
// dspic RB7  
//=============================================================================
void AD9851_FQUD( int value )
{
    if ( value )
        _RB7 = 1;
    else
        _RB7 = 0;
	dly_1us();
}

//=============================================================================
// low level access to LTC2440 chip serial clock bit 
// on parallel port pin 7, DO.5              (SDI data stable on rising edge)
// dspic RF0
//=============================================================================
void LTC2440_SCK( int value )
{
    if ( value )
        _RF0 = 1;
    else
        _RF0 = 0;
	dly_1us();
}

//=============================================================================
// low level access to LTC2440 chip select bit ( low gets 2440's attention )
// on parallel port pin 8, DO.6
// dspic RB8
//=============================================================================
void LTC2440_CS( int value )
{
    if ( value )
        _RB8 = 1;
    else
        _RB8 = 0;
	dly_1us();
}

//=============================================================================
// low level access to LTC2440 serial data out bit (used to read a/d value)
// on parallel port pin 11, +busy input on PC
// note, on the vna this bit is inverted in hardware and on pc... 
// dspic RF6
//=============================================================================
int LTC2440_SDO( void )
{
    if (_RF6)
        return(FALSE);
    else
        return(TRUE);
}

//=============================================================================
// low level access to LTC2440 serial data in bit(used to set conversion speed)
// on parallel port pin 2, D0.0
// This bit is also shared with the DDS data
// dspic 
//=============================================================================
void LTC2440_SDI( int value )
{
    if ( value )
        _RB2 = 1;
    else
		_RB2 = 0;
	dly_1us();
}

//=============================================================================
// low level access to LTC2440 A/D
// This routine tests to see ifthe A/D has finished a conversion and is ready
// to start another or have the results read out.
//=============================================================================
int LTC2440_Ready( void )
{
    int result;

    LTC2440_SCK(0);
    LTC2440_CS(0);
    LTC2440_CS(0);		// test hold cs low for 2us

    if (LTC2440_SDO() == FALSE)     // check for last conversion complete
        result = TRUE;
	else
		result = FALSE;

    LTC2440_CS(1);

    return result;
}

double ADCtoVolts(long int rdg)
{  
    return -(4.6566e-9)*((double)rdg);   // factor is 1.25/(2^28)
}

//=============================================================================
// function to convert frequency to dds tuning word
// done in 64 bit math with rounding
// for best accuracy, refclk and f0 can be passed in in 0.1hz units
//=============================================================================
unsigned long int VNA_FreqToTW(unsigned long int RefClk, unsigned long int F0)
{
	unsigned long long int x;
	x = F0;
	x <<= 32;
	x += (RefClk >> 1);
	x /= RefClk;
	return (unsigned long int) x;
}


//=============================================================================
// Routine to set the rf DDS and the LO DDS to the specified freq and the LO 
// to the specified phase offset.
// local osc phase offset can be 0,90,180 or 270 degrees.
//=============================================================================
void AD9851_SetFreq( unsigned long int hz, unsigned long int phase )
{
	int i;
	unsigned long int divisor;
	divisor =  VNA_FreqToTW(eeb0.refclk,hz);
//	printf("  %u Hz tuning word = %u\n",hz,divisor);

	// set  freq(32) from divisor
	for (i=0; i<32; i++)
	{
		if (divisor & 0x00000001L)
		{
			AD9851_RFDAT(1);
			AD9851_LODAT(1);
		}
		else
		{
			AD9851_RFDAT(0);
			AD9851_LODAT(0);
		}
		AD9851_CLK(1);
		AD9851_CLK(0);
		divisor >>= 1;
	}

	// set direct osc(1), test bit(1),powerup both DDS chips (1)
	AD9851_LODAT(0);
	AD9851_RFDAT(0);
	for (i=0; i<3; i++)
	{
		AD9851_CLK(1);
		AD9851_CLK(0);
	}

	// set zero or 90 deg phase angle (5)
	// 00000 = 0 degrees
	// 01000 = 90 degrees
	// 10000 = 180 degrees
	// 11000 = 270 degrees
	AD9851_RFDAT(0);		// rf is always 0 deg
	for (i=0; i<5; i++)
	{
		if ( ((i == 3) && ((phase == 90)  || (phase == 270))) ||
			 ((i == 4) && ((phase == 180) || (phase == 270))) )
			AD9851_LODAT(1);
		else
			AD9851_LODAT(0);
		AD9851_CLK(1);
		AD9851_CLK(0);
	}
	// now activate the new configuration
	AD9851_FQUD(1);
	AD9851_FQUD(0);
}
//=============================================================================
// Routine to reset the rf DDS and the LO DDS.
// This resets the DDS chips and preps them for receiving 40bit serial 
// command words.
// DO NOT MESS WITH THIS ROUTINE!!!!!!!
// it is very hard to get the DDS's to power up and be synced in a consistant
// configuration. This is critical if saved calibrations are to be used.
// Getting this right took as much time as writting all the other code in
// this project!
//=============================================================================
void AD9851_Reset( void )
{
	int i;
	// set dds lines in a known position
	AD9851_CLK(0);
	AD9851_FQUD(0);
	AD9851_RESET(0);
	AD9851_LODAT(0);
	AD9851_RFDAT(0);

	// reset both dds ( needs to be > 5 ref clk periods )
	AD9851_RESET(1);
	AD9851_RESET(0);

	// enter DDS serial programming mode
	AD9851_CLK(1);
	AD9851_CLK(0);
	AD9851_FQUD(1);
	AD9851_FQUD(0);
	// we are now in serial mode but not synced

	// clock a power down word into 40 bit SR
	for( i=0; i<34; i++)
	{
		AD9851_CLK(1);		// clocks 0..33
		AD9851_CLK(0);
	}
	AD9851_LODAT(1);		// set power down bit
	AD9851_RFDAT(1);
	AD9851_CLK(1);			// clock 34
	AD9851_CLK(0);
	AD9851_LODAT(0);
	AD9851_RFDAT(0);

	for( i=0; i<5; i++)
	{
		AD9851_CLK(1);		// clocks 35..39
		AD9851_CLK(0);
	}

	// reset both dds ( needs to be > 5 ref clk periods )
	AD9851_RESET(1);
	AD9851_RESET(0);

	// clock power down word into core
	AD9851_FQUD(1);
	AD9851_FQUD(0);

	// enter DDS serial programming mode
	AD9851_CLK(1);
	AD9851_CLK(0);
	AD9851_FQUD(1);
	AD9851_FQUD(0);

	// we are now in serial mode, synced and powered down
	// now clock out 40 zeros to power us up
	for( i=0; i<40; i++)
	{
		AD9851_CLK(1);		// clocks 0..39
		AD9851_CLK(0);
	}
	AD9851_FQUD(1);
	AD9851_FQUD(0);

	AD9851_SetFreq( 100000000, 90 );
}

//=============================================================================
// setup port access and init vars for VNA
//=============================================================================
int VNA_Open(void)
{   long int temp;
//    LTC2440_CS(1);          // unselect a/d
//    LTC2440_SCK(0);         // clock low
	// init LTC2410/11/40 into external serial clock mode
	// This is done when sck is low during a H->L transition on CS
    LTC2440_SCK(0);         // clock low
    LTC2440_CS(1);          // unselect a/d
    LTC2440_CS(0);          // select a/d , sets serial mode
    LTC2440_CS(1);          // unselect a/d

	AD9851_Reset();			// get DDS into serial command mode

//	LTC2440_Read(0x0f,NULL);		// dummy read to init a/d
    LTC2440_ReadCDS(&temp);
    return (0); 			// all ok
}

//=============================================================================
// puts vna to sleep ( low power consumption )
// power down DDS chips, turn off status LED turn off encoder LED.
// --------this routine is unused and untested-------
// srategy - call repeatedly, watch a variable incremented by timer
// timer resets variable on keypressed, or serial transaction
// if vraiable exceeds some threshold, power everything possible down and put
// a indication on LCD once a sleep we stay in this routine until a key is pressed
// or a serial byte is received. On wake up, we power up devices and return.
// The tricky partis making sure everyone who calls us will work properly after a power
// down event.
//=============================================================================
//int VNA_Idle(void)
//{
//	int i;
//	// clock a power down word into 40 bit SR
//	for( i=0; i<34; i++)
//	{
//		AD9851_CLK(1);		// clocks 0..33
//		AD9851_CLK(0);
//	}
//	AD9851_LODAT(1);		// set power down bit
//	AD9851_RFDAT(1);
//	AD9851_CLK(1);			// clock 34
//	AD9851_CLK(0);
//	AD9851_LODAT(0);
//	AD9851_RFDAT(0);
//
//	for( i=0; i<5; i++)
//	{
//		AD9851_CLK(1);		// clocks 35..39
//		AD9851_CLK(0);
//	}
//
//	// clock power down word into core
//	AD9851_FQUD(1);
//	AD9851_FQUD(0);
//
//	STATUS_LED = 1;		// led off
//	//ENC_LED = 1;		// turn off encoder led
//
//    return (0); 			// all ok
//}
//
//=============================================================================
//
// Compute One-Port Calibration method error terms from three independent 
// measurements of calibration standards. Open, Short and Load
//
// These three error terms are then used by 
// VNA_ComputeActualReflectionCoefficient() to turn the measurement
// of an unknown into an actual reflection coefficient.
//
// Mo is the measured reflection coefficient for the open calibration standard.
// Ms is the measured reflection coefficient for the short calibration standard.
// Ml is the measured reflection coefficient for the load calibration standard.
//
//=============================================================================
void VNA_ComputeOnePortCalibrationErrorTerms(void)
{  z=-1+2*(Mo-Ml)/(Mo-Ms+1e-38);
   y=(Mo-Ms)*(1-z*z)/2; 
   x=Mo+((Ms-Mo)*(1+z))/2;
   //rho=(Mu-x)/(y+(z*(Mu-x))+1e-38);
}

//=============================================================================
// Computes magnitude of a complex number
//=============================================================================
double my_cabs( __complex__ double x )
{
   return sqrt(( __real__ x * __real__ x) + (__imag__ x * __imag__ x ));
}
//=============================================================================
// Computes argument or angle of a complex number in radians
//=============================================================================
double my_arg( __complex__ double x )
{
   return atan2(__imag__ x, __real__ x);
}

void CalcDUTparms(long int freq, __complex__ double Mu)
{  double f;
   f=(double)(freq/10);
   rho=(Mu-x)/(y+(z*(Mu-x))+1e-38);
   magrho=my_cabs(rho);
   Rerho=__real__ rho;
   Imrho=__imag__ rho;
   RL=-20*log10(magrho+1e-38);
   angrho=(180/M_PI)*my_arg(rho);      // degrees
   Zu=Zo*(1+rho)/(1-rho+1e-38);  // 1e-38 there to avoid div by 0;
   SWR=(1+magrho)/(1-magrho+1e-38);
   Rs=__real__ Zu;
   Xs=__imag__ Zu;
   Q=fabs(Xs)/Rs;
   magZu=my_cabs(Zu);
   angZu=(180/M_PI)*my_arg(Zu);      // degrees
   Ls=Xs/(2.0*M_PI*f);   // H
   Cs=-1/((Xs*2.0*M_PI*f)+1e-38);  // F
   Yu=1/(Zu+1e-38);         // siemens
   magYu=my_cabs(Yu);       // siemens
   angYu=(180/M_PI)*my_arg(Yu);      // degrees
   Gp=__real__ Yu;         // siemens
   Rp=1/(Gp+1e-38);         // ohms
   Xp=-1/(1e-38+__imag__ Yu);    // ohms
   Bp=__imag__ Yu;                   // siemens
   Cp=(__imag__ Yu)/(2.0*M_PI*f);    // F
   Lp=-1/((2.0*M_PI*f*__imag__ Yu)+1e-38);   // H 
}

//=============================================================================
// code to interpolate the calibration constants
// feed it a freq and it will fill in the 3 error terms
// by using quadratic interpolation from the stored cal constants
//=============================================================================
void VNA_interpcalQ( long int fi)
{
	double cif;
	long double step;
        int ci;          // index into calibration structure
	//int rc = 0;
	double y1,y2,y3,A,B,C,f1,f2,f3,f;//,mag,ang;
        //__complex__ double Mo1,Ms1,Ml1;
        //__complex__ double Mo2,Ms2,Ml2;
        //__complex__ double Mo3,Ms3,Ml3;
        __complex__ double xo1,yo1,zo1;
        __complex__ double xo2,yo2,zo2;
        __complex__ double xo3,yo3,zo3;
        __complex__ double mt1,mt2,mt3;
        
	if ( fi < eeb0.f_start )
           fi = eeb0.f_start;
	if ( fi > eeb0.f_stop ) 
           fi = eeb0.f_stop;  
        step = ((long double)(eeb0.f_stop - eeb0.f_start))/(CALPTS-1);
        
        cif=((double)(fi - eeb0.f_start)) / step;
	ci = (int)cif;
   	//printf("f=%ld cif=%f  ci=%i\r\n",fi,cif,ci);
   	//printf("f_start=%ld f_step=%ld\r\n",cal.f_start,cal.f_step);

	while ( ci > (CALPTS-3) )
	   ci--;
	f1=(double)eeb0.f_start+(ci)*step;
	f2=(double)f1+step;
	f3=(double)f2+step;
   	//printf("ci=%i f1=%lf f2=%lf f3=%lf\r\n",ci,f1,f2,f3);
   	f=(double)fi;

        readCAL(ci);
	__real__ Mo = eeb.Mor;
	__imag__ Mo = eeb.Moi;
	__real__ Ms = eeb.Msr;
	__imag__ Ms = eeb.Msi;
	__real__ Ml = eeb.Mlr;
	__imag__ Ml = eeb.Mli;
	__real__ mt1 = eeb.Mtr;
	__imag__ mt1 = eeb.Mti;
	VNA_ComputeOnePortCalibrationErrorTerms();
	xo1=x;
	yo1=y;
	zo1=z;
	
        readCAL(ci+1);
	__real__ Mo = eeb.Mor;
	__imag__ Mo = eeb.Moi;
	__real__ Ms = eeb.Msr;
	__imag__ Ms = eeb.Msi;
	__real__ Ml = eeb.Mlr;
	__imag__ Ml = eeb.Mli;
	__real__ mt2 = eeb.Mtr;
	__imag__ mt2 = eeb.Mti;
	VNA_ComputeOnePortCalibrationErrorTerms();
	xo2=x;
	yo2=y;
	zo2=z;

        readCAL(ci+2);
	__real__ Mo = eeb.Mor;
	__imag__ Mo = eeb.Moi;
	__real__ Ms = eeb.Msr;
	__imag__ Ms = eeb.Msi;
	__real__ Ml = eeb.Mlr;
	__imag__ Ml = eeb.Mli;
	__real__ mt3 = eeb.Mtr;
	__imag__ mt3 = eeb.Mti;
	VNA_ComputeOnePortCalibrationErrorTerms();
	xo3=x;
	yo3=y;
	zo3=z;
	   	
        y1=__real__ xo1;
        y2=__real__ xo2;
        y3=__real__ xo3;
        //y1=my_cabs(xo1);
        //y2=my_cabs(xo2);
        //y3=my_cabs(xo3);
        A=((y3-y2)/((f3-f2)*(f3-f1))) - (y1-y2)/((f1-f2)*(f3-f1));
        B=(A*(f2*f2 - f1*f1) + y1 - y2)/(f1-f2);
        C=y1 - (A*f1*f1 + B*f1);
        __real__(x)=A*f*f + B*f + C;
        //mag=A*f*f + B*f + C;
   	//printf("y1=%f y2=%f y3=%f real(x)=%f\r\n",y1,y2,y3,__real__ x);

        y1=__imag__ xo1;
        y2=__imag__ xo2;
        y3=__imag__ xo3;
        //y1=my_arg(xo1);
        //y2=my_arg(xo2);
        //y3=my_arg(xo3);
        A=((y3-y2)/((f3-f2)*(f3-f1))) - (y1-y2)/((f1-f2)*(f3-f1));
        B=(A*(f2*f2 - f1*f1) + y1 - y2)/(f1-f2);
        C=y1 - (A*f1*f1 + B*f1);
        __imag__(x)=A*f*f + B*f + C;      
        //ang=A*f*f + B*f + C; 
        
        //__real__ x = mag*cos(ang);
        //__imag__ x = mag*sin(ang);     
        
        y1=__real__ yo1;
        y2=__real__ yo2;
        y3=__real__ yo3;
        //y1=my_cabs(yo1);
        //y2=my_cabs(yo2);
        //y3=my_cabs(yo3);
        A=((y3-y2)/((f3-f2)*(f3-f1))) - (y1-y2)/((f1-f2)*(f3-f1));
        B=(A*(f2*f2 - f1*f1) + y1 - y2)/(f1-f2);
        C=y1 - (A*f1*f1 + B*f1);
        __real__(y)=A*f*f + B*f + C;
        //mag=A*f*f + B*f + C;

        y1=__imag__ yo1;
        y2=__imag__ yo2;
        y3=__imag__ yo3;
        //y1=my_arg(yo1);
        //y2=my_arg(yo2);
        //y3=my_arg(yo3);
        A=((y3-y2)/((f3-f2)*(f3-f1))) - (y1-y2)/((f1-f2)*(f3-f1));
        B=(A*(f2*f2 - f1*f1) + y1 - y2)/(f1-f2);
        C=y1 - (A*f1*f1 + B*f1);
        __imag__(y)=A*f*f + B*f + C;      
        //ang=A*f*f + B*f + C; 

        //__real__ y = mag*cos(ang);
        //__imag__ y = mag*sin(ang);     
        
        y1=__real__ zo1;
        y2=__real__ zo2;
        y3=__real__ zo3;
        //y1=my_cabs(zo1);
        //y2=my_cabs(zo2);
        //y3=my_cabs(zo3);
        A=((y3-y2)/((f3-f2)*(f3-f1))) - (y1-y2)/((f1-f2)*(f3-f1));
        B=(A*(f2*f2 - f1*f1) + y1 - y2)/(f1-f2);
        C=y1 - (A*f1*f1 + B*f1);
        __real__(z)=A*f*f + B*f + C;
        //mag=A*f*f + B*f + C;

        y1=__imag__ zo1;
        y2=__imag__ zo2;
        y3=__imag__ zo3;
        //y1=my_arg(zo1);
        //y2=my_arg(zo2);
        //y3=my_arg(zo3);
        A=((y3-y2)/((f3-f2)*(f3-f1))) - (y1-y2)/((f1-f2)*(f3-f1));
        B=(A*(f2*f2 - f1*f1) + y1 - y2)/(f1-f2);
        C=y1 - (A*f1*f1 + B*f1);
        __imag__(z)=A*f*f + B*f + C;      
        //ang=A*f*f + B*f + C; 

        //__real__ z = mag*cos(ang);
        //__imag__ z = mag*sin(ang);     

        y1=__real__ mt1;
        y2=__real__ mt2;
        y3=__real__ mt3;
        A=((y3-y2)/((f3-f2)*(f3-f1))) - (y1-y2)/((f1-f2)*(f3-f1));
        B=(A*(f2*f2 - f1*f1) + y1 - y2)/(f1-f2);
        C=y1 - (A*f1*f1 + B*f1);
        __real__(Mt)=A*f*f + B*f + C;

        y1=__imag__ mt1;
        y2=__imag__ mt2;
        y3=__imag__ mt3;
        A=((y3-y2)/((f3-f2)*(f3-f1))) - (y1-y2)/((f1-f2)*(f3-f1));
        B=(A*(f2*f2 - f1*f1) + y1 - y2)/(f1-f2);
        C=y1 - (A*f1*f1 + B*f1);
        __imag__(Mt)=A*f*f + B*f + C;      
}


//=============================================================================
// code to help give user help in making sure standards are configured ok
//=============================================================================
void Cal_Debug(void)
{       int_freq1(eeb0.f_start);
        GetComplexRdg(eeb0.f_start, 1 );  // br is used for 'break-in'
	sprintf(buff30,"V=%7.5f",my_cabs(Mu));
	lcd_goto(15);
	lcd_puts(buff30);
}

//=============================================================================
// code to generate a calibration file
//=============================================================================
void VNA_calibrate( void )
{
	unsigned long int freq;
        unsigned long int ci;
        long double step;
        //long int ReRdg,ImRdg;

	// bounds check calibration params
	//if (( eeb0.calpts < 2 || eeb0.calpts > 40 ) ||	// validate number of cal points
	if (	( eeb0.f_start < MINFREQ ) ||			// validate minimum freq
		( eeb0.f_start > MAXFREQ) ||
		//( cal.f_step < 10 ) ||					// validate step size
		( eeb0.f_stop > CALSTOP ))
	{
		// setup defaults
		//eeb0.calpts = CALPTS;
		eeb0.f_start = CALSTART;
		eeb0.f_stop = CALSTOP;
		lcd_goto(0x40);
		lcd_puts("Using Defaults...      ");
		while(but3 < 0);	// wait for enter button dwn
		while(but3 > 0);	// wait for enter button up
	}
	//eeb0.cksum=1;
	writeCAL0();

    // do a calibration run and collect data from the 3 OSL standards
    lcd_goto(0x40);
    lcd_puts("Attach open standard    ");
    while(but3 < 0)
    	Cal_Debug();	// wait for enter button dwn
    while(but3 > 0);	// wait for enter button up
    lcd_goto(0x40);
    lcd_puts("      ");
    step = ((long double)(eeb0.f_stop - eeb0.f_start))/(CALPTS-1);

    for (ci = 0; ci < CALPTS; ci++)
    {
        freq = ((long int)(ci * step)) + eeb0.f_start;
        //AD9851_SetFreq( freq, 0 );
        //timer_vna = 100;		// 10ms delay
	lcd_goto(0);
	lcd_puts("CAL                     ");
	int_freq1(freq);
        //while(timer_vna);
        GetComplexRdg(freq, 0);  // br is used for 'break-in'
	eeb.Mor = __real__ Mu;
	eeb.Moi = __imag__ Mu;
	writeCAL(ci);
    }  

    lcd_goto(0x40);
    lcd_puts("Attach short standard   ");
    while(but3 < 0)
    	Cal_Debug();	// wait for enter button dwn
    while(but3 > 0);	// wait for enter button up
    lcd_goto(0x40);
    lcd_puts("      ");

    for (ci = 0; ci < CALPTS; ci++)
    {
        freq = ((long int)(ci * step)) + eeb0.f_start;
        //AD9851_SetFreq( freq, 0 );
        //timer_vna = 100;		// 10ms delay
	lcd_goto(0);
	lcd_puts("CAL                     ");
	int_freq1(freq);
        //while(timer_vna);
	readCAL(ci);
        GetComplexRdg(freq, 0);  // br is used for 'break-in'
	eeb.Msr = __real__ Mu;
	eeb.Msi = __imag__ Mu;
	writeCAL(ci);
    }  

    lcd_goto(0x40);
    lcd_puts("Attach 50 ohm standard  ");
    while(but3 < 0)
    	Cal_Debug();	// wait for enter button dwn
    while(but3 > 0);	// wait for enter button up
    lcd_goto(0x40);
    lcd_puts("      ");

    for (ci = 0; ci < CALPTS; ci++)
    {
        freq = ((long int)(ci * step)) + eeb0.f_start;
        //AD9851_SetFreq( freq, 0 );
        //timer_vna = 100;		// 10ms delay
	lcd_goto(0);
	lcd_puts("CAL                     ");
	int_freq1(freq);
        //while(timer_vna);
	readCAL(ci);
        GetComplexRdg(freq, 0);  // br is used for 'break-in'
	eeb.Mlr = __real__ Mu;
	eeb.Mli = __imag__ Mu;

	if (eeb0.Brdg == 3)    // RFIV
	{  eeb.Mtr = __real__ Vrfiv;             // if RFIV, save the Vsensor data for transmission
	   eeb.Mti = __imag__ Vrfiv;            
	}    
	else                      // Lo-Z or Std
	{  eeb.Mtr = ADCtoVolts(ReRdg);
	   eeb.Mti = ADCtoVolts(ImRdg);
	}
	writeCAL(ci);
    }
    eeb0.cksum = calc_cksum();
    writeCAL0();

    lcd_goto(0x40);
    lcd_goto(0x40);
    lcd_puts("Printing Calibration...");
    printf("Calibration Dump Follows: freq(0.1Hz), r/i: Mo, Ms, Ml, Mt\r\n");
    for (ci = 0; ci < CALPTS; ci++)
    {   readCAL(ci);
        freq = ((long int)(ci * step)) + eeb0.f_start;
	printf("%09ld ",freq);
	printf("%10.3e ",eeb.Mor);
	printf("%10.3e ",eeb.Moi);
	printf("%10.3e ",eeb.Msr);
	printf("%10.3e ",eeb.Msi);
	printf("%10.3e ",eeb.Mlr);
	printf("%10.3e ",eeb.Mli);
	printf("%10.3e ",eeb.Mtr);
	printf("%10.3e\r\n",eeb.Mti);
    }
}

//=============================================================================
// start a conversion and read data from A/D converter 
// this routine is tricky.... see page 15 of the LTC2440 datasheet
// we get here after the last conversion is done and clocked out (but not completely)
// The CS should be low and the SCK should be low.
// returns -1 if error in readings else 0
//=============================================================================
int LTC2440_ReadCDS(long int *rdg)
{
  int i;
  int rc = 0;
  char osr;

  // raising CS aborts last read (we dont need the last sub bit)
  // and forces a new conversion to start
  LTC2440_CS( 1 );

  //if ( volts )		// clear conversion buffer if we got a pointer
  //  *volts = 0L;   
  *rdg=0L;

  // now we wait for the new conversion to finish with a (200ms timeout)
  for ( i=0; i < 200; i++)
  {
    if (LTC2440_Ready() )
	break;
    timerVNAdelay(10);   //  1ms delay
  }

  if ( i >= 200 )
  {
    printf("LTC2440 busy timeout - 200ms. Is board connected and powered?\r\n");
    return( -1 );
  }

  // we got here... data is ready for us to read
  // as we read data, we also clock in the 5 bit config for the next conversion
  LTC2440_CS(0);

  if (eeb0.samplerate == 2)
    osr=15;                // slow osr=11-rate where rate=1 for slowest and 10 for fastest
    //osr=10;                // slow osr=11-rate where rate=1 for slowest and 10 for fastest
    //LTC2440_SDI(1);       // all 1's = slow conversion 6.875 Hz  145 ms
  else if (eeb0.samplerate == 1)
    osr=7;                // medium
  else
    osr=4;                // fast
  //LTC2440_SDI(0);         // all 0's = fast conversion 880 Hz 1.1 ms
  LTC2440_SDI(0);           // OSR4 =0
  dly_1us();
 
  // loop to collect all but 1 bit of result (remember trick?)
  // collect 31 of 32 bits available
  for (i=31; i>=1; i--)
  { 
    LTC2440_SCK(1);
    dly_1us();
    //if ( volts )
    *rdg = (*rdg << 1) + LTC2440_SDO();
    if (i > 27)
    {  if ((osr>>(i-28) & 0x01) == 0)  // this clears/sets the SDI par. port line
          LTC2440_SDI(0);               // clear SDI
       else
          LTC2440_SDI(1);               // set SDI
    } 
    //dly_1us();
    LTC2440_SCK(0);
  }  
  //if ( volts )
  //{
    *rdg = (*rdg << 1);  // compensate for 32nd bit we didnt read above
    // do some sanity checks on conversion and clean up status bits of data
    // top 2 bits should be 0 (/eoc and a 0)
    if (( *rdg & 0xc0000000L) != 0)
    {
      printf("   ***LTC2440 invalid data format 0x%08lX\r\n",*rdg);
      rc = -1;
    }    

    if (( *rdg & 0x30000000L) == 0x30000000L)
    {
      printf("   ***LTC2440 +ve overrange 0x%08lX\r\n",*rdg);
      rc = -1;
    }    
    if (( *rdg & 0x30000000L) == 0L)
    {
      printf("   ***LTC2440 -ve underrange 0x%08lX\r\n",*rdg);
      rc = -1;
    }    
    if ( rc == 0 )
    {
      // good reading... extend sign bit
      *rdg = *rdg << 3;     // puts msbit in sign position
      *rdg = *rdg >> 3;     // leave caller with all the valid bits we collected
    }
  //}
  return rc;    // exit with CS still low... next conversion starts when it goes hi on next call.
}

//=============================================================================
// combined routine to get I and Q voltages at freq
//=============================================================================
char VNA_GetReadingCDS( long int freq, char br)  // br is used for 'break-in'
{
    long int volts0=0, volts90=0, volts180=0, volts270=0;
    
    if ((eeb0.samplerate == 2) && br && ((POSCNT!=POSCNTlast) || (but1>0) || (but2>0) || (but3>0)) )
    {   POSCNTlast=POSCNT;
        return (1);
    }
    AD9851_SetFreq( freq,0 );
    timerVNAdelay(10);   //  1ms delay
    LTC2440_ReadCDS(&volts0);
    
    if ((eeb0.samplerate == 2) && br && ((POSCNT!=POSCNTlast) || (but1>0) || (but2>0) || (but3>0)))
    {   POSCNTlast=POSCNT;
        return (1);
    }
    AD9851_SetFreq( freq,180 );
    timerVNAdelay(10);   //  1ms delay
    LTC2440_ReadCDS(&volts180);
    
    if ((eeb0.samplerate == 2) && br && ((POSCNT!=POSCNTlast) || (but1>0) || (but2>0) || (but3>0)))
    {   POSCNTlast=POSCNT;
        return (1);
    }
    AD9851_SetFreq( freq,90 );
    timerVNAdelay(10);   //  1ms delay
    LTC2440_ReadCDS(&volts90);
    
    if ((eeb0.samplerate == 2) && br && ((POSCNT!=POSCNTlast) || (but1>0) || (but2>0) || (but3>0)))
    {   POSCNTlast=POSCNT;
        return (1);
    }
    AD9851_SetFreq( freq,270 );
    timerVNAdelay(10);   //  1ms delay
    LTC2440_ReadCDS(&volts270);
    
    ReRdg=(volts0 - volts180) >> 1;
    ImRdg=(volts90 - volts270) >>1;
    return (0);
}

char GetComplexRdg(long int freq, char br)  // br is used for 'break-in'
{   if (eeb0.Brdg == 3)      // RFIV
    {  Vsensor();
       //printf("POSCNT=%i\r\n",POSCNT);
       if (VNA_GetReadingCDS(freq, br ))
	  return (1);  
       __real__ Vrfiv = ADCtoVolts(ReRdg);            
       __imag__ Vrfiv = ADCtoVolts(ImRdg);              
       Isensor();
       if (VNA_GetReadingCDS(freq,  br ))
	  return 1;  
       __real__ Irfiv = ADCtoVolts(ReRdg);            
       __imag__ Irfiv = ADCtoVolts(ImRdg);              
       Mu = Vrfiv/(Irfiv + 1e-38);  
    }    
    else if (eeb0.Brdg == 2)   // Hi-Z
    {  Isensor();
       if (VNA_GetReadingCDS(freq, br ))
	  return (1);  
       __real__ Mu = ADCtoVolts(ReRdg);
       __imag__ Mu = ADCtoVolts(ImRdg);
    }
    else                       // Lo-Z or Std
    {  Vsensor();
       if (VNA_GetReadingCDS(freq, br ))
	  return (1);  
       __real__ Mu = ADCtoVolts(ReRdg);
       __imag__ Mu = ADCtoVolts(ImRdg);
    }
    return (0);
}
