/* int_cab.c  
 * dsPic30f4011 driver program for the N2PK Vector Network Analyzer
 * This file contains code for the operators interface and calcs for cable parms.
 *
 *  Copyright (C) 2010 By Paul Kiciak N2PK
 *             (paul@n2pk.com)
 *
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Compile using: mplab ide
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>		// for PI etc
#include "vna.h"

extern volatile int but1;				// -DEBOUNCE to +DEBOUNCE
extern volatile int but2;
extern volatile int but3;

extern volatile unsigned short int timer_test;
extern volatile unsigned short timer_led;
extern void AD9851_SetFreq( unsigned long int hz, unsigned long int phase );

extern char buff30[];		// global text buffer for screen i/o

extern void lcd_goto( int );
extern void lcd_puts( char * );
extern char VNA_GetReadingCDS( long int freq, char br);  // br is used for 'break-in'
extern void CalcDUTparms(long int freq, __complex__ double Mu);
extern double ADCtoVolts(long int rdg);
										 
extern void int_SWR( double swr );

//extern void int_freq1( unsigned long freq );
extern void VNA_interpcalQ( long int freq );
extern double my_cabs( __complex__ double x );
extern double my_arg( __complex__ double x );
extern char GetComplexRdg(long int freq, char br);  // br is used for 'break-in'

//static char data[15];		// local buffer for numeric conversions to string

extern __complex__ double Mo,Ms,Ml,Mt,x,y,z;
extern long lastreffreq;
extern long lasttrnfreq;

extern double Rs,Xs,Cs,Ls;				
extern double Rp,Xp,Gp,Bp,Cp,Lp;			
extern double Q;					
extern double SWR,RL;					
extern double magrho,angrho,Rerho,Imrho;
extern double magZu,angZu,magYu,angYu;		
       
       // unknown reading
extern __complex__ double Mu;		        // raw voltages
extern __complex__ double rho;			// computed reflection coeff.
extern __complex__ double Zu,Yu;			// computed impedance
extern __complex__ double Zo;

struct EEbuffer0                         // 1st row (32 bytes)
{  long int refclk;			 // 156.25 MHz clk
   unsigned long int f_start, f_stop;	 // freq in 0.1 Hz for calibration points
   unsigned long int lastreffreq, lasttrnfreq;
   unsigned long int lastdelf;
   long int cksum;
   char samplerate;			 // a sample rate index: 0,1,2
   char Brdg;                             // 0=Std, 1=Lo-Z, 2=Hi-Z, 3=RFIV
   char lastresultrefnum,lastresulttrnnum;
};

extern char Brdg;    // 0=Std, 1=Lo-Z, 2=Hi-Z, 3=RFIV

extern __complex__ double Vrfiv,Irfiv;

extern struct EEbuffer0 eeb0;

extern void readCAL0(void);  // reads the first 32 bytes in EEPROM 
extern void writeCAL0(void);  // writes the first 32 bytes in EEPROM 

extern void Vsensor(void);
extern void Isensor(void);
extern volatile unsigned long int timer_auto_off; 

char resultcabnum;

double Mrholeft,Angleft,Releft,Imleft,Zmin,Zmax;  //,Zminmax,Angminmax;
double Mrhoright,Angright,Reright,Imright;
long Fleft,Fright,freq;  //,Fminmax,Fmin,Fmax,fstep;

#define FMIN 30000000L   // 3 MHz

double Zc,T,K1;  // cable parameters

double Re0,Re1,Re2,Re3,Im0,Im1,Im2,Im3;
double F0,F1,F2,F3;
double Mrho0,Mrho1,Mrho2,Mrho3,Ang0,Ang1,Ang2,Ang3;
extern long int ReRdg,ImRdg;
static char data[15];		// local buffer for numeric conversions to string

//char FindZminmax(unsigned long fs)
//// This looks for the frequency at which angle(Z) is zero. That generally corresponds to |Z|min or |Z|max.
//// returns 2 & Zminmax Fminmax if Angle(Z)=0 is found and |Z| is max
////         1 & Zminmax Fminmax if Angle(Z)=0 is found and |Z| is min
////         0 if no solution found
//// enters with components of rho @ f, and f which is the starting point for the
//// search
//// for the leftmost axis crossing, the following criteria are made 
////     1. imag(rho) at f is negative
////     2. clockwise rotation to Fleft where 0<imag(rho)<0.0001
////        and Fleft > fs 
//{   double AngO, AngN, ZminO, ZminN;
//    long fold, fnew;
//    //long int ReRdg, ImRdg;
//    int i;
//    i=0;
//    fold=fs;
//    GetComplexRdg(freq, 0);
//    VNA_interpcalQ(freq); 
//    CalcDUTparms(freq, Mu); 
//    ZminO=magZu;
//    AngO=angZu;  
//    //if (AngO <= 0)
//    //   AngO += 360;  
//    //printf("Searching for min |Z|:\r\n");
//    freq=fs + 1000000L;  // increment fs by 100 kHz 
//    Lfzmin1:
//    i++;
//    if (freq > eeb0.f_stop)
//       freq=eeb0.f_stop;
//    if (freq < eeb0.f_start)
//       freq = eeb0.f_start;
//    if (freq < FMIN)
//       freq = FMIN;  
//    //if (freq < fs)
//    //   freq = fs;
//    
//    GetComplexRdg(freq, 0);
//    VNA_interpcalQ(freq); 
//    CalcDUTparms(freq, Mu);   
//    ZminN=magZu;
//    AngN=angZu;  // degrees
//    //if (AngN <= 0)
//    //   AngN += 360; 
//    //printf("i=%i Freq=%ld AngO=%.2f AngN=%.2f\r\n",i,freq,AngO,AngN);
//    fnew=freq;
//    //printf(".");
//    //printf("angle(rho%i) = %-.2f at %-.6f MHz\n",i,AngN,fnew);
//    if (((fabs((ZminO-ZminN)/ZminN)) < 0.001)  && ((fabs(AngN) < 0.01)))  // converged
//    {  Fminmax = freq; 
//       //printf("Converged after %i iterations: |Zmin|=%-.4f AngZ=%-.4f at Fmin=%-.6f MHz\r\n\n",i,ZminN,AngN,((double)Fmin)/1e7);
//       Zminmax=ZminN;
//       Angminmax=AngN;  // degrees
//       freq=freq+10*fstep;
//       GetComplexRdg(freq, 0);
//       VNA_interpcalQ(freq); 
//       CalcDUTparms(freq, Mu);   
//       if (magZu > Zminmax)
//          return (1);    // Zminmax is min
//       else
//          return (2);    // Zminmax is max
//    }
//    else
//    {  if (i > 99)
//       {  //printf("\nLast RHO%i = %-.4f + j%-.4f at F=%-.6f MHz\n",i,ReN,ImN,fnew);
//          return 0;
//       }
//       fstep = ((long int)((AngN-0)*((double)(fnew-fold))/(AngO-AngN+1e-38)));
//       if (fstep > 2000000L)
//          fstep=2000000L;
//       if (fstep < -2000000L)
//          fstep = -2000000L;
//       freq = fnew + fstep;
//       //printf("freq=%ld fnew=%ld fold=%ld AngN=%.3f AngO=%.3f\r\n",freq,fnew,fold,AngN,AngO);
//       ZminO = ZminN;
//       AngO = AngN;
//       fold = fnew;
//       goto Lfzmin1;
//    }
//}

char FindLeftRho(unsigned long fs,double rer,double imr)
// This looks for the leftmost crossing of the Rn=0 crossing on the Smith Chart
// returns i & Mrholeft,Angleft,Releft,Imleft,Fleft if leftmost axis crossing is found
//         0 if not
// enters with components of rho @ f, and f which is the starting point for the
// search
// for the leftmost axis crossing, the following criteria are made 
//     1. imag(rho) at f is negative
//     2. clockwise rotation to Fleft where 0<imag(rho)<0.0001
//        and Fleft > fs 
{   double AngO, AngN, ReO, ReN, ImO, ImN;
    long fold, fnew;
    //long int ReRdg, ImRdg;
    int i;
    i=0;
    fold=fs;
    ReO=rer;
    ImO=imr;
    __real__ rho=ReO;
    __imag__ rho=ImO;
    AngO=(180/M_PI)*my_arg(rho);  // degrees
    if (AngO <= 0)
       AngO += 360;  
    printf("Searching for leftmost Rn=0 crossing:\r\n");
    freq=fs + 1000000L;  // increment fs by 100 kHz
    Lflr1:
    //printf("angle(rho%i) = %-.2f at %-.6f MHz\n",i,AngO,fold);
    i++;
    if (freq > eeb0.f_stop)
       freq=eeb0.f_stop;
    //if (freq < eeb0.f_start)
    //   freq = eeb0.f_start;
    //if (freq < FMIN)
    //   freq = FMIN;  
    if (freq < fs)
       freq = fs;
    
    GetComplexRdg(freq, 0);
    VNA_interpcalQ(freq); 
    CalcDUTparms(freq, Mu);   
    //rho=(Mu-Xm)/(Ym+(Zm*(Mu-Xm))+1e-38);
    ReN=__real__ rho;
    ImN=__imag__ rho;
    AngN=angrho;  // degrees
    if (AngN <= 0)
       AngN += 360; 
    //printf("i=%i Freq=%ld AngO=%.2f AngN=%.2f\r\n",i,freq,AngO,AngN);
    fnew=freq;
    //printf(".");
    //printf("angle(rho%i) = %-.2f at %-.6f MHz\n",i,AngN,fnew);
    if ((fnew > fs) && (ImN < 0.00005) && (ImN > -0.00005))  // converged
    {  Fleft = freq; 
       //printf("Converged after %i iterations: Rho = %-.4f + j%-.4f at Fleft=%-.6f MHz\r\n\n",i,ReN,ImN,((double)Fleft)/1e7);
       Releft=ReN;
       Imleft=ImN;
       Mrholeft=magrho;
       Angleft=angrho;  // degrees
       Zmin = magZu;
       return i;
    }
    else
    {  if (i > 29)
       {  //printf("\nLast RHO%i = %-.4f + j%-.4f at F=%-.6f MHz\n",i,ReN,ImN,fnew);
          Zmin=0;
          return 0;
       }
       freq = fnew + ((long int)((AngN-180.0)*((double)(fnew-fold))/(AngO-AngN+1e-38)));
       //printf("freq=%ld fnew=%ld fold=%ld AngN=%.3f AngO=%.3f\r\n",freq,fnew,fold,AngN,AngO);
       ImO = ImN;
       ReO = ReN;
       AngO = AngN;
       fold = fnew;
       goto Lflr1;
    }
}

char FindRightRho(unsigned long fs,double rer,double imr)
// This looks for the rightmost crossing of the Rn=0 crossing on the Smith Chart
// returns i & Mrhoright,Angright,Reright,Imright,Fright if rightmost axis crossing is found
//         0 if not
// enters with components of rho @ f, and f which is the starting point for the
// search.
// for the rightmost axis crossing, the following criteria are made 
//     1. imag(rho) at fs is positive
//     2. clockwise rotation to Fright where -0.0005<imag(rho)<0
//        and Fright > fs 
{   double AngO, AngN, ReO, ReN, ImO, ImN;
    long fold, fnew;
    //long int ReRdg, ImRdg;
    int i;
    i=0;
    fold=fs;
    ReO=rer;
    ImO=imr;
    __real__ rho=ReO;
    __imag__ rho=ImO;
    AngO=(180/M_PI)*my_arg(rho);  // degrees
    //if (AngO < 0)
    //   AngO = 360+AngO;
    printf("Searching for rightmost Rn=0 crossing:\r\n");
    freq=fs + 1000000L;  // increment fs by 100 kHz
 
    Lfrr1:
    //printf("angle(rho%i) = %-.2f at %-.6f MHz\n",i,AngO,fold);
    i++;
    if (freq > eeb0.f_stop)
       freq=eeb0.f_stop;
    //if (freq < eeb0.f_start)
    //   freq = eeb0.f_start;
    //if (freq < FMIN)
    //   freq = FMIN;     
    if (freq < fs)
       freq = fs;
    //printf("i=%i Freq=%ld\r\n",i,freq);
       
    GetComplexRdg(freq, 0);
    VNA_interpcalQ(freq); 
    CalcDUTparms(freq, Mu);   

    ReN=__real__ rho;
    ImN=__imag__ rho;
    AngN=angrho;  // degrees
    //if (AngN < 0)
    //   AngN = 360+AngN;
    //printf("i=%i Freq=%ld AngO=%.2f AngN=%.2f\r\n",i,freq,AngO,AngN);
    fnew=freq;
    //printf(".");
    //printf("angle(rho%i) = %-.2f at %-.6f MHz\n",i,AngN,fnew);
    if ((fnew > fs) && (ImN < 0.00005) && (ImN > -0.00005))  // converged
    {  Fright = freq; 
       //printf("Converged after %i iterations: Rho = %-.4f + j%-.4f at Fright=%-.6f MHz\r\n\n",i,ReN,ImN,((double)Fright/1e7));
       Reright=ReN;
       Imright=ImN;
       Mrhoright=magrho;
       Angright=angrho;  // degrees
       Zmax=magZu;
       return i;
    }
    else
    {  if (i > 29)
       {  //printf("\nLast RHO%i = %-.4f + j%-.4f at F=%-.6f MHz\n",i,ReN,ImN,fnew);
          Zmax=0;
          return 0;
       }
       freq = fnew + ((long int)((AngN-0.0)*((double)(fnew-fold))/(AngO-AngN)));
       //printf("freq=%ld fnew=%ld fold=%ld AngN=%.3f AngO=%.3f\r\n",freq,fnew,fold,AngN,AngO);
       ImO = ImN;
       ReO = ReN;
       AngO = AngN;
       fold = fnew;
       goto Lfrr1;
    }
}
    
char FindLineT(void)
// returns 1 & T if t-line time of flight is found
//         0 if not found (no actual returns this way...
// starts the search from the lowest cal freq, f1 (1 MHz) or FMIN whichever is higher.
// If angle(rho,f1) is negative, it looks for the first leftmost 
// crossing of rho with Rn=0. Designate that F1eft. Then search from there for 
// the next rightmost crossing of Rn=0. Designate that Fright.
// If angle(rho,f1) is positive at the start, then the search order for 
// Fright comes first, followed by Fleft.
// F1 is the first one found, F2 is the second one found.
// Estimated T = 250/(F2-F1) where the F's are in MHz & T is ns.
// If one or both of Fleft or Fright is not found, then T=1.3889*delta(degrees)/delta(MHz) in ns is 
// using Fright or Fleft if found, or F0, or F3. This won't be as accurate an estimate when the
// the cable Zo doesn't match the center of the Smith Chart.
// Capitalized variable suffixes are used as follows:
//    0 - values at f1 (lowest cal freq),
//    1 - values at first Rn=0 axis crossing (if found),
//    2 - values at second Rn=0 axis crossing (if found),
//    3 - values at f2 (highest cal freq).
// "delta(degrees)" must also account for Rn=0 axis crossings, if present, if using
// either F0 or F3 data.
//
{   char F1found, F2found;  // each are iter # if corresponding freq is found, 0 if not.
    //long int ReRdg, ImRdg;
    //printf("\nSearching for transmission line delay (T)\n");
    //Lflt0:  
    
    freq = eeb0.f_start;
    if (freq < FMIN)
       freq = FMIN;
    GetComplexRdg(freq, 0);
    
    VNA_interpcalQ(freq); 
    CalcDUTparms(freq, Mu);   
    Mrho0=magrho;
    Re0=__real__ rho;
    Im0=__imag__ rho;
    Ang0=angrho;  // degrees
    F0=((double)freq)/1e7;
 
    freq=eeb0.f_stop;  // get the data at the highest cal frequency
    GetComplexRdg(freq, 0);
 
    VNA_interpcalQ(freq); 
    CalcDUTparms(freq, Mu);   
    Mrho3=magrho;
    Ang3=angrho;      // degrees
    Re3=__real__ rho;
    Im3=__imag__ rho;
    F3=((double)freq)/1e7;
    
    //printf("F0=%.3f Ang0=%.3f\r\n", F0,Ang0);
    //printf("F3=%.3f Ang3=%.3f\r\n", F3,Ang3);
    
    if (Ang0 < 0)     // Ang0 is negative, search for Fleft first
    {  if (0 != (F1found = FindLeftRho((unsigned long)1e7*F0,Re0,Im0)))
       {  Mrho1 = Mrholeft;
          Ang1 = fabs(Angleft);
          Re1 = Releft;
	  Im1 = Imleft;
          F1 = ((double)Fleft)/1e7;
          if (0 != (F2found = FindRightRho(Fleft,Re1,Im1)))
          {  F2 = ((double)Fright)/1e7;
             Mrho2= Mrhoright;
             Ang2= Angright;
             Re2 = Reright;
             Im2 = Imright;
             T = 250/(F2-F1);   // T in ns
             printf("T=%06.3f F1=%-.3f F2=%-.3f\r\n", T, F1, F2);
             return(1);
          }
          else  // F2 not found - use F0 and F1 for T calc (Ang0 & Ang1 are opposite sign, Ang1 > 0)
          {  T = 1.3889*(Ang1-Ang0)/(F1-F0);    // T in ns   
             //printf("T=%06.3f F1=%-.3f F0=%-.3f Ang1=%-.3f Ang0=%-.3f\r\n", T, F1, F0, Ang1, Ang0 );
             return(1);
          }
       }
       else  // F1 not found, so don't search for F2 and just calc T using F0 and F3
       {  T = 1.3889*(Ang0-Ang3)/(F3-F0);     // T in ns   
          return(1);
       }
    }
    else  // Ang0 is positive - search for Fright first
    {  if (0 != (F1found = FindRightRho((unsigned long)1e7*F0,Re0,Im0)))
       {  Mrho1 = Mrhoright;
          Ang1 = Angright;
          Re1 = Reright;
          Im1 = Imright;
          F1 = ((double)Fright)/1e7;
          if (0 != (F2found = FindLeftRho(Fright,Re1,Im1)))
	  {  F2 = ((double)Fleft)/1e7;
             Mrho2= Mrholeft;
             Ang2= Angleft;
             Re2 = Releft;
             Im2 = Imleft;
	     if ((F2 + (F2-F1)) < F3)  // try again for Fright
	     {  //Ang2 = Angright;
	     	if (0 != (F1found = FindRightRho((unsigned long)1e7*F2,Re2,Im2)))
                {  Mrho1 = Mrhoright;
                   Ang1 = Angright;
                   Re1 = Reright;
                   Im1 = Imright;
                   F1 = ((double)Fright)/1e7;
                   T = 250/(F1-F2);   // T in ns
                   printf("T=%06.3f F1=%-.3f F2=%-.3f\r\n", T, F1, F2);
                   return(1);
                }
             }
             T = 250/(F2-F1);   // T in ns
             printf("T=%06.3f F1=%-.3f F2=%-.3f\r\n", T, F1, F2);
             return(1);
          }
          else  // F2 not found - use F0 and F1 for T calc (Ang0 & Ang1 are opposite sign, Ang0 > 0, Ang1 < 0)
          {  T = 1.3889*(Ang0-Ang1)/(F1-F0);  // T in ns   
             return(1);
          }
       }
       else  // F1 not found, so don't search for F2 and just calc T using F0 and F3
       {  T = 1.3889*(Ang0-Ang3)/(F3-F0);     // T in ns   
          return(1);
       }
    }
}  

//__complex__ double ctanh(__complex__ double z)
//{   __complex__ double result;
//    //double denom;
//    __real__ result = sinh(2.0*__real__ z);
//    __imag__ result = sin(2.0*__imag__ z);
//    result = result/((cosh(2.0*__real__ z)) + (cos(2.0*__imag__ z)) + 1e-38);
//    return (result);
//}

////=============================================================================
// low level screen access to display and manage the cab screen
//	on this screen, only the enter button is used
//=============================================================================
void int_cab( void )
{       char result=0, resultlast=1; //, iret1, iret2;
	lcd_goto(0);
	lcd_puts("Attach (one) cable, open");
        lcd_goto(0x40);
        lcd_puts("or 25 ohms at far end.  ");
        while(but3 < 0);        // wait for enter button dwn
        while(but3 > 0);	// wait for enter button up
        
	lcd_goto(0);
        lcd_puts("Measuring...            ");
        lcd_goto(0x40);
        lcd_puts("                        ");
        FindLineT();
        Zc=sqrt(Zmin*Zmax);
        //printf("T=%-.3fns\r\n", T);
        
        //printf("Zo=%.3f\r\n\n",Zc);
        //Adjust Mrho0, Mrho3, and K1 for measured cable Zo vs. rho measured in 50 ohm system:
        if (Zc != 0)
        {  __real__ rho = Re3;
           __imag__ rho = Im3;
           Zu = Zo * (1+rho)/(1-rho+1e-38);
           rho = (Zu - Zc)/(Zu + Zc + 1e-38);
           Mrho3 = my_cabs(rho);
           //printf("Zc=%-.2f Zur=%-.2f Zui=%-.2f Mrho3=%-.4f\r\n",Zc,__real__ Zu, __imag__ Zu, Mrho3);
           __real__ rho = Re0;
           __imag__ rho = Im0;
           Zu = Zo * (1+rho)/(1-rho+1e-38);
           rho = (Zu - Zc)/(Zu + Zc + 1e-38);
           Mrho0 = my_cabs(rho);
        }
        K1=-0.5*(log(Mrho3))/sqrt(F3);  // nepers do least squares fit later or add a 2nd screen with db at 2 f's. 
        K1=8.6859*K1;   // dB
        //printf("K1=%.4fdB/sqrt(MHz)\r\n\n",K1);
        while (1)
	{     if ( but1 > 0 )
	      {	 timer_auto_off=AUTOOFFCNT;   // auto power-off
		 result--;
		 if ( result < 0 )
		    result = 2;
		    while(but1 > 0);		// wait for button release
		    timerLEDdelay(2000);
	      }
	      if ( but2 > 0 )
	      {	 timer_auto_off=AUTOOFFCNT;   // auto power-off
		 result++;
		 if ( result > 2 )
		    result = 0;
		    while(but2 > 0);		// wait for button release
		    timerLEDdelay(2000);
	      }
	      if ( but3 > 0 )
	      {  timer_auto_off=AUTOOFFCNT;   // auto power-off
		 while( but3 > 0 );		// wait for button release
		 timerLEDdelay(5000);
		 break;
	      }
	      if (result != resultlast)
	      {  resultlast = result;
	         switch (result)
                 {
                 case 0:  lcd_goto(0);
	                  //sprintf(buff30,"CAB Zo=%06.2f T=%06.3fns",Zc,T);
	                  if (Zc == 0)
	                     sprintf(buff30,"CAB Zo=NoCalc T=");
	                  else
	                     sprintf(buff30,"CAB Zo=%06.2f T=",Zc);	            
                          if (fabs(T) >= 1000)
	                     sprintf(data,"%06.4fus",T/1000);
	                  else if (fabs(T) >=100)
	                     sprintf(data,"%06.2fns",T);
                          else if (fabs(T) >=10)
	                     sprintf(data,"%06.3fns",T);
                          else if (fabs(T) >=1)
	                     sprintf(data,"%06.4fns",T);
	                  else
	                     sprintf(data,"%06.2fps",1000*T);
	                  strcat(buff30,data);
	                  lcd_puts(buff30);
                          lcd_goto(0x40);
	                  sprintf(buff30,"K=%07.4fdB/sqrt(MHz)   ",K1);
	                  lcd_puts(buff30);
	                  break;
	         case 1:  lcd_goto(0);
	                  sprintf(buff30,"CAB %06.3fdB @ %06.3fMHz",(double)(-10.0*log10(Mrho0)),F0);
	                  lcd_puts(buff30);
                          lcd_goto(0x40);
	                  sprintf(buff30,"Att %06.3fdB @ %06.3fMHz",(double)(-10.0*log10(Mrho3)),F3);
	                  lcd_puts(buff30);
	                  break;
	         case 2:  if (Zc!=0)
	                  {  lcd_goto(0);
	                     sprintf(buff30,"CAB %07.3f%c @ %06.3fMHz",Zmin,0xf4,((double)Fleft)/1e7);
	                     lcd_puts(buff30);
                             lcd_goto(0x40);
	                     sprintf(buff30,"|Z| %07.2f%c @ %06.3fMHz",Zmax,0xf4,((double)Fright)/1e7);
	                     lcd_puts(buff30);
	                  }
	                  else
	                  {  lcd_goto(0);
	                     sprintf(buff30,"CAB     No Zmin         ");
	                     lcd_puts(buff30);
                             lcd_goto(0x40);
	                     sprintf(buff30,"|Z|     No Zmax         ");
	                     lcd_puts(buff30);
	                  }
	                  break;
	         }
	      }
	      
	}
}

