/* int_cal.c  
 * dsPic30f4011 driver program for the N2PK Vector Network Analyzer
 * This file contains code for the operators interface.
 *
 *  Copyright (C) 2006 By Lawrence Glaister VE7IT
 *             (ve7it@shaw.ca)
 *
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Compile using: mplab ide
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>		// for PI etc
#include "vna.h"

extern volatile int but1;				// -DEBOUNCE to +DEBOUNCE
extern volatile int but2;
extern volatile int but3;

extern volatile unsigned short int timer_test;
extern volatile unsigned short timer_led;
extern void AD9851_SetFreq( unsigned long int hz, unsigned long int phase );
extern double my_cabs( __complex__ double x );
extern char VNA_GetReadingCDS( long int freq, char br);  // br is used for 'break-in'
extern void int_cmplx( __complex__ num );
extern void CalcDUTparms(long int freq, __complex__ double Mu);
extern void VNA_interpcalQ( long int freq);
extern double my_cabs( __complex__ double x );
extern char GetComplexRdg(long int freq, char br);  // br is used for 'break-in'

// to conserve stack space define a general use char buffer as global
extern char buff30[30];

extern void lcd_goto( int );
extern void lcd_puts( char *);
extern void int_freq1( long int );
extern void int_freq1cal( long int );
extern void int_freq2( long int );
extern void VNA_calibrate( void );
extern int save_cal( void );
extern int restore_cal( void );
extern double ADCtoVolts(long int rdg);

extern __complex__ double Mo,Ms,Ml,Mt,x,y,z;

extern double Rs,Xs,Cs,Ls;				
extern double Rp,Xp,Gp,Bp,Cp,Lp;			
extern double Q;					
extern double SWR,RL;					
extern double magrho,angrho,Rerho,Imrho;
extern double magZu,angZu,magYu,angYu;		
       
       // unknown reading
extern __complex__ double Mu;		        // raw voltages
extern __complex__ double rho;			// computed reflection coeff.
extern __complex__ double Zu,Yu;			// computed impedance
extern __complex__ double Zo;

struct EEbuffer0                         // 1st row (32 bytes)
{  long int refclk;			 // 156.25 MHz clk
   unsigned long int f_start, f_stop;	 // freq in 0.1 Hz for calibration points
   unsigned long int lastreffreq, lasttrnfreq;
   unsigned long int lastdelf;
   long int cksum;
   char samplerate;			 // a sample rate index: 0,1,2
   char Brdg;                             // 0=Std, 1=Lo-Z, 2=Hi-Z, 3=RFIV
   char lastresultrefnum,lastresulttrnnum;
};

extern char Brdg;    // 0=Std, 1=Lo-Z, 2=Hi-Z, 3=RFIV

extern __complex__ double Vrfiv,Irfiv;

//extern struct EEbuffer eeb;

extern struct EEbuffer0 eeb0;

extern void readCAL0(void);  // reads the first 32 bytes in EEPROM 
extern void writeCAL0(void);  // writes the first 32 bytes in EEPROM 

extern void Vsensor(void);
extern void Isensor(void);

extern long int lastreffreq;
extern long int lasttstfreq;
extern long int lasttrnfreq;
extern volatile unsigned long int timer_auto_off; 
extern long int ReRdg,ImRdg;
extern char resultrefnum, resulttrnnum;

//=============================================================================
// This is a low level routine that checks the current calibration iterpolation.
// It gets called right after calibration while the 50ohm standard is still 
// attached and checks freqs across the range to make sure we get values 
// within 1%. If calibration plane is at the end of a piece of coax, the 
// cal iterpolation may fail due to rapidily changing cal constants around
// frequencies where the coax becomes a resonant length.
//=============================================================================
void test_cal(void)
{
	unsigned long int freq,freqi;
	unsigned long int i;
	double step;

	double reference, test;

        reference = my_cabs(Zo);

	//fmax = cal.f_start + ((cal.f_num -1)*cal.f_step);
	lcd_goto(0x40);
	lcd_puts("Checking Cal Interp.....");
	while( timer_test );
        step = ((long double)(eeb0.f_stop - eeb0.f_start))/(CALPTS-1);
	//for ( freq = cal.f_start; freq < fmax; freq += cal.f_step )  
	for ( i=0; i<(CALPTS-1); i++)
	{  freq = eeb0.f_start + i*step;
	    // read hardware and convert to requested display
	   freqi=freq + (long int)step/2;
	   int_freq1(freqi);		
	   VNA_interpcalQ(freqi);
	   GetComplexRdg(freqi, 0);

   	   CalcDUTparms(freqi, Mu);
    	   test = my_cabs(Zu);
           if ( (test > (reference * 1.01)) || (test < (reference * 0.99)))
           {
               // oops... maybe a suspect cal.
               lcd_goto(0x40);
               lcd_puts("Suspect Cal Interpolate ");
               printf("Suspect cal interpolation for load is %.1fohms at %ldHz\r\n",test,freqi/10);
               timer_test = 20000;        // 2 sec delay
               while( timer_test );
               lcd_goto(0x40);
               lcd_puts("                        ");               
           }
        }
}

//=============================================================================
// low level screen access to display and manage the calibration screen
// 
//=============================================================================
void int_cal( void )
{
	unsigned long freq1 = eeb0.f_start;
	unsigned long freq2 = eeb0.f_stop;
	int fieldnum = 0;		// local indicator of which field is adjustable
	long int encoder;
        #define NUMFLD 4
	const int fieldpos[NUMFLD] = {6,17,0x42,0x55};

	lcd_goto(0);
	lcd_puts("CAL                     ");
	lcd_goto(0x40);
        sprintf(buff30,"Abort  CalPts=%03i  Start",CALPTS);

	lcd_puts(buff30);
	int_freq2(freq2);
	int_freq1cal(freq1);	

	POSCNT = 0;			// reset encoder counter

	while (1)
	{
		// read buttons to see if we need to do something
		// previous button
		if ( but1 > 0 )
		{	timer_auto_off=AUTOOFFCNT;   // auto power-off
			fieldnum--;
			if (fieldnum < 0) fieldnum = NUMFLD -1;
			lcd_goto(fieldpos[fieldnum]);
			while(but1 > 0);		// wait for button release
		}
		// next button
		if ( but2 > 0 )
		{       timer_auto_off=AUTOOFFCNT;   // auto power-off
			fieldnum++;
			if (fieldnum >= NUMFLD) fieldnum = 0;
			lcd_goto(fieldpos[fieldnum]); 	
			while(but2 > 0); 		//wait for button release
		}
		// enter button
		if ( but3 > 0 )
		{       timer_auto_off=AUTOOFFCNT;   // auto power-off
			while( but3 > 0 );			// wait for button release
			if ( fieldnum == 2) break;	// abort calibration
			if ( fieldnum ==3 )			// start cal
			{
				if (freq1 < freq2)
				{
					eeb0.f_start = freq1;
					//eeb0.calpts = CALPTS;
					eeb0.f_stop = freq2;
					eeb0.lastreffreq = eeb0.lasttrnfreq = freq1;
					eeb0.lastdelf=10000L;  // 1kHz default
					eeb0.lastresultrefnum=eeb0.lastresulttrnnum=0;  // don't do Brdg here - handled in CFG
					writeCAL0();
					VNA_calibrate();		// collect standard data
					lcd_goto(0x40);
					lcd_puts("Writing Cal to EEPROM...");
					timer_test = 10000;		// 1 sec delay
					while( timer_test );
					test_cal();			// see if cal interp works across freq range
					lastreffreq=eeb0.lastreffreq;
					lasttrnfreq=eeb0.lasttrnfreq;
					resultrefnum=eeb0.lastresultrefnum;
					resulttrnnum=eeb0.lastresulttrnnum;
					break;
				}
				else
				{
					lcd_goto(0x40);
					lcd_puts("Error: F1 must be < F2  ");
					timer_test = 30000;		// 3 sec delay
					while( timer_test );
					lcd_goto(0x40);
					lcd_puts("Abort              Start");
					fieldnum = 0;		// put cursor back on 1st number
					lcd_goto(fieldpos[fieldnum]);
				}
			}
		}

		// check for relative encoder activity
		// note encoder res is halved to make it easier to set freq
		encoder = (long)POSCNT << 16;
		encoder = encoder >> 16;		// crude sign extention
		encoder /= 2;					// reduce control sensitivity
		if (encoder)
		{       timer_auto_off=AUTOOFFCNT;   // auto power-off
			POSCNT = 0;		// reset encoder
			if ( fieldnum == 0 )
			{
				freq1 += encoder * 250000L;
				if ( freq1 < MINFREQ ) freq1 = MAXFREQ;
				if ( freq1 > MAXFREQ ) freq1 = MINFREQ;
				//if ( freq1 > freq2 ) freq1 = freq2;
				int_freq1cal(freq1);
			}
			if ( fieldnum == 1 )
			{
				freq2 += encoder * 250000L;
				//if ( freq2 < freq1 ) freq2 = freq1;
				if ( freq2 > MAXFREQ ) freq2 = MINFREQ;
				if ( freq2 < MINFREQ ) freq2 = MAXFREQ;
				int_freq2(freq2);
			}
		}			
		// let outside world know we are alive (1hz flash)
		if ( timer_led == 0 )
		{
			//TODO read hardware to display voltages
			timer_led = 5000;			// 500ms to next update
			STATUS_LED = ~STATUS_LED;
		}
	}
}
