/* int_cfg.c  
 * dsPic30f4011 driver program for the N2PK Vector Network Analyzer
 * This file contains code for the operators interface.
 *
 *  Copyright (C) 2006 By Lawrence Glaister VE7IT
 *             (ve7it@shaw.ca)
 *
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Compile using: mplab ide
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>		// for PI etc
#include "vna.h"

extern volatile int but1;				// -DEBOUNCE to +DEBOUNCE
extern volatile int but2;
extern volatile int but3;

extern volatile unsigned short int timer_test;
extern volatile unsigned short timer_led;
extern void AD9851_SetFreq( unsigned long int hz, unsigned long int phase );

// to conserve stack space define a general use char buffer as global
extern char buff30[30];

extern void lcd_goto( int );
extern void lcd_puts( char * );
extern int save_cal( void );
//extern void set_pwm( unsigned int );

struct EEbuffer0                         // 1st row (32 bytes)
{  long int refclk;			 // 156.25 MHz clk
   unsigned long int f_start, f_stop;	 // freq in 0.1 Hz for calibration points
   unsigned long int lastreffreq, lasttrnfreq;
   unsigned long int lastdelf;
   long int cksum;
   char samplerate;			 // a sample rate index: 0,1,2
   char Brdg;                             // 0=Std, 1=Lo-Z, 2=Hi-Z, 3=RFIV
   char lastresultrefnum,lastresulttrnnum;
};

extern struct EEbuffer0 eeb0;

extern void writeCAL0(void);  // writes the first 32 bytes in EEPROM 

extern char Brdg;    // 0=Std, 1=Lo-Z, 2=Hi-Z, 3=RFIV
extern volatile unsigned long int timer_auto_off; 

//=============================================================================
// low level screen access to display and manage the config screen
// 
//=============================================================================
void int_cfg( void )
{
	static int fieldnum;		// local indicator of which field is adjustable
	long int encoder;
        #define NUMFLD 5
	const int fieldpos[NUMFLD] = {0x42,0x45,0x48,0x4a,0x4f}; // screen posns for cursor	
	char samplerate;
	char *rates[] = {"Fast","Med ","Slow"};
	char *brdg[] = {"STD ","Lo-Z","Hi-Z","RFIV"};

	fieldnum = 3;		// default entry at detector rate

	// read freq, contrast and sample rate from nonvolatile memory
	// and init local vars
	samplerate = eeb0.samplerate;
	Brdg = eeb0.Brdg;
        // |  Refclk  Rate Brdg    |
        // |nnnnnnnnn Slow RFIV    |  Brdg field is "RFIV", "Hi-Z", "Lo-Z", or "STD "

	lcd_goto(0);
	lcd_puts("  RefClk  Rate Brdg     ");
	lcd_goto(0x40);
	lcd_puts("                        ");

	lcd_goto(0x40);
	sprintf(buff30,"%9ld ",eeb0.refclk/10);
	lcd_puts(buff30);

	lcd_goto(fieldpos[3]);
	lcd_puts(rates[eeb0.samplerate]);

	lcd_goto(fieldpos[4]);
	lcd_puts(brdg[eeb0.Brdg]);
	lcd_goto(fieldpos[fieldnum]);

	POSCNT=0;

	while (1)
	{
		// read buttons to see if we need to do something
		// previous button
		if ( but1 > 0 )
		{	timer_auto_off=AUTOOFFCNT;   // auto power-off
			fieldnum--;
			if ( fieldnum < 0 )
				fieldnum = NUMFLD - 1;
			lcd_goto(fieldpos[fieldnum]);
			while(but1 > 0);		// wait for button release
		}
		// next button
		if ( but2 > 0 )
		{       timer_auto_off=AUTOOFFCNT;   // auto power-off
			fieldnum++;
			if ( fieldnum >= NUMFLD )
				fieldnum = 0;
			lcd_goto(fieldpos[fieldnum]);
			while(but2 > 0); 		//wait for button release
		}

   		// enter button
		if ( but3 > 0 )
		{       timer_auto_off=AUTOOFFCNT;   // auto power-off
			while( but3 > 0 );		// wait for button release
			// write freq, contrast and sample rate to 
			// nonvolatile memory from  local vars
			eeb0.samplerate = samplerate;
			//save_cal();
			//Brdg /= 25;
			if (eeb0.Brdg != Brdg)
			{  lcd_goto(0x40);
                           lcd_puts("New Cal is required     ");
			   eeb0.Brdg=Brdg;							
                           timer_test = 20000;        // 2 sec delay
                           while( timer_test );
                           lcd_goto(0x40);
                           lcd_puts("                        ");               
                        }
			writeCAL0();
			break;
		}
		// check for relative encoder activity
		encoder = (long)POSCNT << 16;
		encoder = encoder >> 16;		// crude sign extention
		encoder /= 32;
		if (encoder)
		{       timer_auto_off=AUTOOFFCNT;   // auto power-off
			POSCNT = 0;		// reset encoder
			switch (fieldnum)
			{
			case 0:		// vna ref osc freq (around 150mhz)
			case 1:
			case 2:
				// three tuning rates supported
				if ( fieldnum == 0) 
					eeb0.refclk += (encoder * 10000000);	// 1 MHz
				if ( fieldnum == 1 )
					eeb0.refclk += (encoder * 10000);		// 1 kHz
				if ( fieldnum == 2 )
					eeb0.refclk += (encoder * 10);			// Hz
				
				// I am assuming user has used a ref oscillator between these
				// ranges.... if too far out, then a recompile is required
				if ( eeb0.refclk < 1400000000L ) eeb0.refclk = 1400000000L;
				if ( eeb0.refclk > 1800000000L ) eeb0.refclk = 1800000000L;

				// reset vna freq to 10mhz using new ref osc freq
				// this will let one zero beat WWV or 10mhz std osc
				sprintf(buff30,"%9ld",eeb0.refclk/10);
				lcd_goto(0x40);				
				lcd_puts(buff30);
				// set freq using new ref osc.. zero beat with 10mhz wwv 
				AD9851_SetFreq( 100000000L, 0L );
				break;
			case 3:		// analog to digital sample rate
				// to reduce sensitivity... it takes 10 enc pulses to bump to next speed
                                samplerate += encoder;
				if (samplerate < 0) samplerate = 2;
				if (samplerate > 2) samplerate = 0;
				lcd_puts(rates[samplerate]);
				break;
			case 4:		// RFIV Test head Control
				Brdg += encoder;
				if (Brdg < 0) Brdg = 3;
				if (Brdg > 3) Brdg = 0;
	                        //lcd_goto(fieldpos[5]);
	                        lcd_puts(brdg[Brdg]);
				break;
			}
			// restore cursor to begiining of active field
			lcd_goto(fieldpos[fieldnum]);
		}			
		// let outside world know we are alive (1hz flash)
		if ( timer_led == 0 )
		{
			timer_led = 100;			// 10ms to next update
			STATUS_LED = ~STATUS_LED;
		}
	}
}
