/* int_main.c  
 * dsPic30f4011 driver program for the N2PK Vector Network Analyzer
 * This file contains code for the operators interface.
 *
 *  Copyright (C) 2006 By Lawrence Glaister VE7IT
 *             (ve7it@shaw.ca)
 *
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Compile using: mplab ide
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>		// for PI etc
#include "vna.h"

extern volatile int but1;				// -DEBOUNCE to +DEBOUNCE
extern volatile int but2;
extern volatile int but3;

extern volatile unsigned short int timer_test;
extern volatile unsigned short timer_led;
extern void AD9851_SetFreq( unsigned long int hz, unsigned long int phase );

// to conserve stack space define a general use char buffer as global
char buff30[30];

extern void lcd_goto( int );
extern void lcd_puts( char * );
extern void VNA_Open( void );
extern int restore_cal( void );
extern void int_run( void );
extern void int_trn( void );
extern void int_cal( void );
extern void int_cfg( void );
extern void int_tst( void );
extern void int_rmt( void );
extern void int_sdb( void );
extern void int_batt( void );

struct EEbuffer0                         // 1st row (32 bytes)
{  long int refclk;			 // 156.25 MHz clk
   unsigned long int f_start, f_stop;	 // freq in 0.1 Hz for calibration points
   unsigned long int lastreffreq, lasttrnfreq;
   unsigned long int lastdelf;
   long int cksum;
   char samplerate;			 // a sample rate index: 0,1,2
   char Brdg;                             // 0=Std, 1=Lo-Z, 2=Hi-Z, 3=RFIV
   char lastresultrefnum,lastresulttrnnum;
};

char Brdg;    // 0=Std, 1=Lo-Z, 2=Hi-Z, 3=RFIV

__complex__ double Vrfiv,Irfiv;

struct EEbuffer                          // rows after first - also 32 bytes each
{  double Mor,Moi;                     // open
   double Msr,Msi;                     // short
   double Mlr,Mli;                     // load
   double Mtr,Mti;                     // thru
};

extern struct EEbuffer0 eeb0;
extern struct EEbuffer eeb;

extern void readCAL0(void);  // reads the first 32 bytes in EEPROM 
extern void writeCAL0(void);  // writes the first 32 bytes in EEPROM 
void readCAL(long int ci);  // ci ranges 0...(CALPTS-1) and is an index to a freq's worth of cal data 

extern void Vsensor(void);
extern void Isensor(void);
extern volatile unsigned long int timer_auto_off; 

extern char resultrefnum, resulttrnnum;

extern void int_cab(void);
extern void int_minswr(void);
extern void timerLEDdelay(int delay);

//=============================================================================
// Stuff that may need to change for your hardware environment
//=============================================================================

//=============================================================================
// local defines.... you should not have to mess with these
//=============================================================================
#define	TRUE	(1)
#define	FALSE	(0)	

//=============================================================================
// local global vars used by this program
//=============================================================================
//	extern  int fieldnum;		// local indicator of which field is adjustable

//=============================================================================
// low level screen access to display and manage the main screen
// 
//=============================================================================

//=============================================================================
// Routine to calculate a checksum on the cal data section of EEPROM
// call with array size in 16 bit words and ptr to start.
//=============================================================================
long int calc_cksum(void)
{    int i,j;
     long unsigned int cksum = 0;
     //long int *ptr = (long int *)&eeb;
     long int *ptr = &eeb;

     for (i=0; i < CALPTS; i++)
     {   readCAL(i);              // read 32 bytes of cal data from EEPROM
         for (j=0; j < 8; j++)
         {   cksum += *ptr++;
             //printf("i=%02i j=%02i ptr=%04x &ptr=%09ld cksum=%09ld\r\n",i,j,ptr,&ptr,cksum);
             //printf("i=%02i j=%02i ptr=%04x cksum=%10lu\r\n",i,j,ptr,cksum);
             //ptr += 1;         
         }
         ptr = ptr - 8;
     }
     printf("cksum=%10lu\r\n",cksum);
     return cksum;
}

void int_more(void)
{    static int fieldnum = 0;		// local indicator of which field is adjustable
     int pos[]={0,4,8,21};
     STATUS_LED = 1;					// led off

     lcd_goto(0);
     lcd_puts("MORE  VE7IT/N2PK  0.000v");
     lcd_goto(0x40);
     lcd_puts("Tst Rmt Dbg          Ret");
     
     while (1)
     {	  // read buttons to see if we need to do something
     	  // previous button
     	  if ( but1 > 0 )
     	  {  timer_auto_off=AUTOOFFCNT;   // auto power-off
     	     fieldnum--;
     	     if ( fieldnum < 0 )
     	        fieldnum = 3;
     	     lcd_goto(pos[fieldnum] + 0x40);
     	     while(but1 > 0);		// wait for button release
     	     timerLEDdelay(1000);
     	  }
     	  // next button
     	  if ( but2 > 0 )
     	  {  timer_auto_off=AUTOOFFCNT;   // auto power-off
     	     fieldnum++;
     	     if ( fieldnum > 3 )
     	        fieldnum = 0;
     	     lcd_goto(pos[fieldnum] + 0x40);
     	     while(but2 > 0); 		//wait for button release
     	     timerLEDdelay(1000);
     	  }
          
     	  // enter button
     	  if ( but3 > 0 )
     	  {  timer_auto_off=AUTOOFFCNT;   // auto power-off
     	     while( but3 > 0 );		// wait for button release
     	     switch(fieldnum)
     	     { 	case 0:
     	  	     int_tst();
     	  	     break;
     	  	case 1:
     	  	     int_rmt();
     	  	     break;
     	  	case 2:
     	  	     int_sdb();
     	  	     break;
     	  	case 3:
     	  	     return;         
     	     }
     	     // after we return from a mode, redisplay main screen
     	     // using last cursor position
     	     lcd_goto(0);
     	     lcd_puts("MORE  VE7IT/N2PK  0.000v");
     	     lcd_goto(0x40);
             lcd_puts("Tst Rmt Dbg          Ret");
     	     lcd_goto(pos[fieldnum] + 0x40);
     	  }
     	  // led outside world know we are alive (1hz flash)
     	  // and update battery voltage display
     	  if ( timer_led == 0 )
     	  {
     	  	timer_led = 10000;
     	  	STATUS_LED = ~STATUS_LED;
     	  	int_batt();			// update battery voltage
     	  	lcd_goto(pos[fieldnum] + 0x40);
     	  }
     }
}

void int_swp(void)
{    static int fieldnum = 0;		// local indicator of which field is adjustable
     int pos[]={0,4,21};
     STATUS_LED = 1;					// led off

     lcd_goto(0);
     lcd_puts("SWP   VE7IT/N2PK  0.000v");
     lcd_goto(0x40);
     lcd_puts("Cab SWR              Ret");
     
     while (1)
     {	  // read buttons to see if we need to do something
     	  // previous button
     	  if ( but1 > 0 )
     	  {  timer_auto_off=AUTOOFFCNT;   // auto power-off
     	     fieldnum--;
     	     if ( fieldnum < 0 )
     	        fieldnum = 2;
     	     lcd_goto(pos[fieldnum] + 0x40);
     	     while(but1 > 0);		// wait for button release
     	     timerLEDdelay(1000);
     	  }
     	  // next button
     	  if ( but2 > 0 )
     	  {  timer_auto_off=AUTOOFFCNT;   // auto power-off
     	     fieldnum++;
     	     if ( fieldnum > 2 )
     	        fieldnum = 0;
     	     lcd_goto(pos[fieldnum] + 0x40);
     	     while(but2 > 0); 		//wait for button release
     	     timerLEDdelay(1000);
     	  }
          
     	  // enter button
     	  if ( but3 > 0 )
     	  {  timer_auto_off=AUTOOFFCNT;   // auto power-off
     	     while( but3 > 0 );		// wait for button release
     	     switch(fieldnum)
     	     { 	case 0:
     	  	     int_cab();
     	  	     break;
     	  	case 1:
     	  	     int_minswr();
     	  	     break;
     	  	case 2:
     	  	     return;         
     	     }
     	     // after we return from a mode, redisplay main screen
     	     // using last cursor position
     	     lcd_goto(0);
     	     lcd_puts("SWP   VE7IT/N2PK  0.000v");
     	     lcd_goto(0x40);
             lcd_puts("Cab SWR              Ret");
     	     lcd_goto(pos[fieldnum] + 0x40);
     	     timerLEDdelay(5000);
     	  }
     	  // led outside world know we are alive (1hz flash)
     	  // and update battery voltage display
     	  if ( timer_led == 0 )
     	  {
     	  	timer_led = 10000;
     	  	STATUS_LED = ~STATUS_LED;
     	  	int_batt();			// update battery voltage
     	  	lcd_goto(pos[fieldnum] + 0x40);
     	  }
     }
}

//======================================================

//======================================================
void int_main( void )
{
	static int fieldnum = 0;		// local indicator of which field is adjustable
	timer_led = 10000;				// 1.0 seconds
	//int cs;
	int pos[]={0,4,8,12,16,20};
	long int cksumr;

	STATUS_LED = 1;					// led off

	lcd_goto(0);
	lcd_puts("SVNA  VE7IT/N2PK  0.000v");
	// size of cal structure must be <= 1024 bytes
	// to fit in eeprom
	printf("Calsize=%04d bytes.\r\n",32+CALPTS*32);  // EEPROM: 32 bytes of common vars first row, CALPTS of OSLT data @ 32 bytes each

	// Open the VNA.
	VNA_Open();

	lcd_goto(0x40);
        //lcd_puts("Ref Trn Cal Cfg Ts Rm Db");
        lcd_puts("Ref Swp Trn Cal Cfg More");
        //lcd_puts("Ref Cab Cal Cfg Ts Rm Db");
	lcd_goto(pos[fieldnum] + 0x40);

	readCAL0();
	cksumr = calc_cksum();
	
	if (eeb0.cksum != cksumr)
	{  // opps, no valid calibration detected
	   // assume we are starting from a new box
	   eeb0.refclk = 1562500000L;
	   eeb0.samplerate = 2;
	   //eeb0.calpts = CALPTS;
	   eeb0.f_start = CALSTART;
	   eeb0.f_stop = CALSTOP;
           eeb0.Brdg=0;
	   lcd_goto(0x40);
	   lcd_puts("  Please Cfg & Cal Me   ");
	   //printf("No valid calibration found in EEPROM\r\n");
	   fieldnum = 4;		// force default to cfg screen
	}
	else
	{  Brdg = eeb0.Brdg;
	   resultrefnum = eeb0.lastresultrefnum;   
	   resulttrnnum = eeb0.lastresulttrnnum;
	}   
	AD9851_SetFreq( 100000000L, 0L );
		
	while (1)
	{
		// read buttons to see if we need to do something
		// previous button
		if ( but1 > 0 )
		{	timer_auto_off=AUTOOFFCNT;   // auto power-off
			fieldnum--;
			if ( fieldnum < 0 )
			   fieldnum = 5;
			lcd_goto(pos[fieldnum] + 0x40);
			while(but1 > 0);		// wait for button release
		}
		// next button
		if ( but2 > 0 )
		{       timer_auto_off=AUTOOFFCNT;   // auto power-off
			fieldnum++;
			if ( fieldnum > 5 )
			   fieldnum = 0;
			lcd_goto(pos[fieldnum] + 0x40);
			while(but2 > 0); 		//wait for button release
		}

   		// enter button
		if ( but3 > 0 )
		{       timer_auto_off=AUTOOFFCNT;   // auto power-off
			while( but3 > 0 );		// wait for button release
			switch(fieldnum)
			{
			case 0:
				int_run();
				fieldnum=0;
				break;
			case 1:
				int_swp();
				break;
			case 2:
				int_trn();
				break;
			case 3:
				int_cal();
				break;
			case 4:
				AD9851_SetFreq( 100000000L, 0L );		
                                int_cfg();
				break;
			case 5:
				int_more();
				break;

			}
			// after we return from a mode, redisplay main screen
			// using last cursor position
			lcd_goto(0);
			lcd_puts("SVNA  VE7IT/N2PK  0.000v");
			lcd_goto(0x40);
                        lcd_puts("Ref Swp Trn Cal Cfg More");
			lcd_goto(pos[fieldnum] + 0x40);
		}
		// led outside world know we are alive (1hz flash)
		// and update battery voltage display
		if ( timer_led == 0 )
		{
			timer_led = 10000;
			STATUS_LED = ~STATUS_LED;
			int_batt();			// update battery voltage
			lcd_goto(pos[fieldnum] + 0x40);
		}
	}
}
