/* int_rmt.c  
 * dsPic30f4011 driver program for the N2PK Vector Network Analyzer
 * This file contains code for the operators interface.
 *
 *  Copyright (C) 2006 By Lawrence Glaister VE7IT
 *             (ve7it@shaw.ca)
 *
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Compile using: mplab ide
 * for some reason, this file pukes the compiler when using -O2,3 or s
 */
// 30Nov2006  -- rearranged remote so less chance of loosing serial bytes

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>		// for PI etc
#include "vna.h"

extern volatile int but1;				// -DEBOUNCE to +DEBOUNCE
extern volatile int but2;
extern volatile int but3;

extern volatile unsigned short int timer_test;
extern volatile unsigned short timer_led;
extern void AD9851_SetFreq( unsigned long int hz, unsigned long int phase );

// to conserve stack space define a general use char buffer as global
extern char buff30[30];

extern void lcd_goto( int );
extern void lcd_puts( char * );
extern char VNA_GetReadingCDS( long int freq, char br);  // br is used for 'break-in'
extern void int_cmplx( __complex__ num );
extern void CalcDUTparms(long int freq, __complex__ double Mu);
extern void VNA_interpcalQ( long int freq);
extern double ADCtoVolts(long int rdg);
extern char GetComplexRdg(long int freq, char br);  // br is used for 'break-in'

extern __complex__ double Mo,Ms,Ml,Mt,x,y,z;

extern double Rs,Xs,Cs,Ls;				
extern double Rp,Xp,Gp,Bp,Cp,Lp;			
extern double Q;					
extern double SWR,RL;					
extern double magrho,angrho,Rerho,Imrho;
extern double magZu,angZu,magYu,angYu;		
       
       // unknown reading
extern __complex__ double Mu;		        // raw voltages
extern __complex__ double rho;			// computed reflection coeff.
extern __complex__ double Zu,Yu;			// computed impedance
extern __complex__ double Zo;

struct EEbuffer0                         // 1st row (32 bytes)
{  long int refclk;			 // 156.25 MHz clk
   unsigned long int f_start, f_stop;	 // freq in 0.1 Hz for calibration points
   unsigned long int lastreffreq, lasttrnfreq;
   unsigned long int lastdelf;
   long int cksum;
   char samplerate;			 // a sample rate index: 0,1,2
   char Brdg;                             // 0=Std, 1=Lo-Z, 2=Hi-Z, 3=RFIV
   char lastresultrefnum,lastresulttrnnum;
};

extern char Brdg;    // 0=Std, 1=Lo-Z, 2=Hi-Z, 3=RFIV

extern __complex__ double Vrfiv,Irfiv;

extern struct EEbuffer0 eeb0;

extern void Vsensor(void);
extern void Isensor(void);
extern volatile unsigned long int timer_auto_off; 
extern long int ReRdg,ImRdg;

//=============================================================================
// low level screen access to display and manage the remote cpu screen
// 
//=============================================================================
void int_rmt( void )
{       //long int ReRdg=0, ImRdg=0;
	char buff[10];
	int	buff_index;				// serial interface freq string buff
	char ch;

	unsigned long int freq = 35000000L;	// current freq requested by pc
	//unsigned long int fmax;
	long int serialcnt = 0;
	//ReRdg=0;
	//ImRdg=0;

	lcd_goto(0);
	lcd_puts("Rmt 12.245MHz I/O=12345 ");
	lcd_goto(0x40);
	lcd_puts("Zu=                     ");

	buff_index = 0;
	buff[0] = 0;

	// clear out serial buffer
	while(U1STAbits.URXDA) 
		ch=U1RXREG;


	while (1)
	{
		// TODO interact with serial port
		//  receive a freq, compute impedance and send it back

		if (U1STAbits.URXDA)
		{
			// if a char is waiting, holdoff updating local screen
			timer_led = 5000;			// 500ms to next update
			serialcnt++;
			ch = U1RXREG & 0xFF;
			// char available from pc put it in buffer if its a number
			if ( buff_index < (sizeof(buff) - 2) && 
				ch >= '0' &&
				ch <= '9')
			{
				buff[buff_index] = ch;
				buff[buff_index + 1] = 0;
				buff_index++;
			}

			if ( ch == 0x0d || ch == 0x0a )
			{       timer_auto_off=AUTOOFFCNT;   // auto power-off
				// end of line, see if buffer has a frequency for us
				// we should have at least a 4 digit number in the buffer
				freq = atol( buff );
				freq=freq*10;      // so we don't have to enter 0.1 Hz
				if ( freq < eeb0.f_start ) 
					freq = eeb0.f_start;
				if ( freq > eeb0.f_stop ) 
					freq = eeb0.f_stop;

				// read hardware and convert to requested display
				VNA_interpcalQ(freq);
 	                        GetComplexRdg(freq, 0);
   				CalcDUTparms(freq, Mu); 
 
				// send real and imag parts back to pc
				printf("%ldHz %f %f\r\n",freq/10,Rs, Xs);

				// reset input stream
				buff_index = 0;
				buff[0] = 0;
			}
		}

   		// enter button
		if ( but3 > 0 )
		{       timer_auto_off=AUTOOFFCNT;   // auto power-off
			while( but3 > 0 );		// wait for button release
			break;
		}

		// let outside world know we are alive (1hz flash)
		// and update serial status displays
		if ( timer_led == 0 )
		{
			timer_led = 5000;			// 500ms to next update
			STATUS_LED = ~STATUS_LED;
			lcd_goto(0);                           //place cursor just left of "Hz"
			sprintf(buff30,"Rmt %9sHz %08ld",buff,serialcnt);
     		if (U1STAbits.URXDA) continue;
			lcd_puts(buff30);			// display current freq and cnt
     		if (U1STAbits.URXDA) continue;
			lcd_goto(0x43);				// display current impedance
     		if (U1STAbits.URXDA) continue;
			int_cmplx(Zu);
     		if (U1STAbits.URXDA) continue;
			//lcd_goto(24);				// put cursor off screen
			lcd_goto(12);                           //place cursor just left of "Hz"
		}
	}
}
