/* int_run.c  
 * dsPic30f4011 driver program for the N2PK Vector Network Analyzer
 * This file contains code for the operators interface.
 *
 *  Copyright (C) 2006 By Lawrence Glaister VE7IT
 *             (ve7it@shaw.ca)
 *
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Compile using: mplab ide
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>		// for PI etc
#include "vna.h"

extern volatile int but1;				// -DEBOUNCE to +DEBOUNCE
extern volatile int but2;
extern volatile int but3;

extern volatile unsigned short int timer_test;
extern volatile unsigned short timer_led;
extern void AD9851_SetFreq( unsigned long int hz, unsigned long int phase );

extern char buff30[];		// global text buffer for screen i/o

extern void lcd_goto( int );
extern void lcd_puts( char * );
extern char VNA_GetReadingCDS( long int freq, char br);  // br is used for 'break-in'
extern void CalcDUTparms(long int freq, __complex__ double Mu);
extern double ADCtoVolts(long int rdg);
extern char GetComplexRdg(long int freq, char br);  // br is used for 'break-in'
										 
extern void int_SWR( double swr );

extern void int_freq1( long int );
extern void int_freq1cal( long int );
extern void int_freq2( long int );

extern void VNA_interpcalQ( long int freq );
extern double my_cabs( __complex__ double x );
extern double my_arg( __complex__ double x );

static char data[15];		// local buffer for numeric conversions to string

//extern int fieldnum;		// local indicator which field gets encoder adj

extern __complex__ double Mo,Ms,Ml,Mt,x,y,z;
extern long lastreffreq;
extern long lasttrnfreq;
extern long lastdelf;
extern long lastminswrfreq1;
extern long lastminswrfreq2;

extern double Rs,Xs,Cs,Ls;				
extern double Rp,Xp,Gp,Bp,Cp,Lp;			
extern double Q;					
extern double SWR,RL;					
extern double magrho,angrho,Rerho,Imrho;
extern double magZu,angZu,magYu,angYu;		
       
       // unknown reading
extern __complex__ double Mu;		        // raw voltages
extern __complex__ double rho;			// computed reflection coeff.
extern __complex__ double Zu,Yu;			// computed impedance
extern __complex__ double Zo;

struct EEbuffer0                         // 1st row (32 bytes)
{  long int refclk;			 // 156.25 MHz clk
   unsigned long int f_start, f_stop;	 // freq in 0.1 Hz for calibration points
   unsigned long int lastreffreq, lasttrnfreq;
   unsigned long int lastdelf;
   long int cksum;
   char samplerate;			 // a sample rate index: 0,1,2
   char Brdg;                             // 0=Std, 1=Lo-Z, 2=Hi-Z, 3=RFIV
   char lastresultrefnum,lastresulttrnnum;
};

extern char Brdg;    // 0=Std, 1=Lo-Z, 2=Hi-Z, 3=RFIV

extern __complex__ double Vrfiv,Irfiv;

extern struct EEbuffer0 eeb0;

extern void readCAL0(void);  // reads the first 32 bytes in EEPROM 
extern void writeCAL0(void);  // writes the first 32 bytes in EEPROM 

extern void Vsensor(void);
extern void Isensor(void);
extern volatile unsigned long int timer_auto_off; 

char resultrefnum, resulttrnnum;
extern long int ReRdg,ImRdg;

double tg, gdb;
long delf;
long Fswrmin;

extern void timerLEDdelay(int delay);

double FindMinSWR(long f1,long f5)
// This looks for the minimum |rho| between f1 and f2
// returns min |rho| and Fminrho
{   double swrmin,swr1,swr2,swr3,swr4,swr5;
    //long freq, fstep;
    long fstep,f2,f3,f4;
    int i;
    
    GetComplexRdg(f1, 0);
    VNA_interpcalQ(f1); 
    CalcDUTparms(f1, Mu);   
    swr1=fabs(SWR);
    
    GetComplexRdg(f5, 0);
    VNA_interpcalQ(f5); 
    CalcDUTparms(f5, Mu);   
    swr5=fabs(SWR);
    
    fstep=(f5-f1)/4;
    f3=(f1+f5)/2;
    GetComplexRdg(f3, 0);
    VNA_interpcalQ(f3); 
    CalcDUTparms(f3, Mu);
    swr3=fabs(SWR); 
         
    printf("Searching for minimum SWR:\r\n");
    for(i=0; i<15; i++)
    {  f2=f3-fstep;
       f4=f3+fstep;  
       GetComplexRdg(f2, 0);
       VNA_interpcalQ(f2); 
       CalcDUTparms(f2, Mu);
       swr2=fabs(SWR); 
        
       GetComplexRdg(f4, 0);
       VNA_interpcalQ(f4); 
       CalcDUTparms(f4, Mu);
       swr4=fabs(SWR);
         
       swrmin=swr3;
       Fswrmin=f3;
       if (swr2<swrmin)
       {  swrmin=swr2;
          Fswrmin=f2;
       }
       if (swr4<swrmin)
       {  swrmin=swr4;
          Fswrmin=f4;
       }
       //printf("Min SWR= %-.4f f2=%ld f3=%ld f4=%ld fstep=%ld\r\n",swrmin,f2,f3,f4,fstep); 
       swr3=swrmin;
       f3=Fswrmin;
       fstep /= 2;
    }
    if (swrmin>swr1)
    {  swrmin=swr1;
       Fswrmin=f1;
    }
    else if (swrmin>swr5)
    {  swrmin=swr5;
       Fswrmin=f5;
    }
    printf("Min SWR= %-.4f @ %ld\r\n",swrmin,Fswrmin); 
    return (swrmin);
}

void int_minswr(void)
{       double swrmin;
	unsigned long freq1;
	unsigned long freq2;
	int fieldnum = 0;		// local indicator of which field is adjustable
	long int encoder;
        #define NUMFLDSWR 4
	const int fieldpos[NUMFLDSWR] = {6,17,0x42,0x55};
	
	if (lastminswrfreq1)
           freq1=lastminswrfreq1;
        else
           freq1=eeb0.f_start;
           
	if (lastminswrfreq2)
           freq2=lastminswrfreq2;
        else
           freq2=eeb0.f_stop;

	lcd_goto(0);
	lcd_puts("REF                     ");
	lcd_goto(0x40);
        sprintf(buff30,"Abort   Min.SWR   Search");

	lcd_puts(buff30);
	int_freq2(freq2);
	int_freq1cal(freq1);	

	POSCNT = 0;			// reset encoder counter

	while (1)
	{
		// read buttons to see if we need to do something
		// previous button
		if ( but1 > 0 )
		{	timer_auto_off=AUTOOFFCNT;   // auto power-off
			fieldnum--;
			if (fieldnum < 0) fieldnum = NUMFLDSWR -1;
			lcd_goto(fieldpos[fieldnum]);
			while(but1 > 0);		// wait for button release
		}
		// next button
		if ( but2 > 0 )
		{       timer_auto_off=AUTOOFFCNT;   // auto power-off
			fieldnum++;
			if (fieldnum >= NUMFLDSWR) fieldnum = 0;
			lcd_goto(fieldpos[fieldnum]); 	
			while(but2 > 0); 		//wait for button release
		}
		// enter button
		if ( but3 > 0 )
		{       timer_auto_off=AUTOOFFCNT;   // auto power-off
			while( but3 > 0 );			// wait for button release
			if ( fieldnum == 2) break;	// abort search for min SWR
			if ( fieldnum ==3 )			// start search for min swr
			{       lcd_goto(0x40);
			        lcd_puts("Searching for min SWR...");
			        swrmin=FindMinSWR(freq1,freq2);
				lcd_goto(0x40);
				if (swrmin<1000)
				   sprintf(buff30,"SWR=%+08.3f @ %06.3fMHz",swrmin,(double)Fswrmin/1e7);
				else
				   sprintf(buff30,"|SWR| > 1000 @ %06.3fMHz",(double)Fswrmin/1e7);
	                        lcd_puts(buff30);
				//timer_test = 10000;		// 1 sec delay
				//while( timer_test );
                                while(but3 < 0);        // wait for enter button dwn
                                while(but3 > 0);	// wait for enter button up
				lastminswrfreq1=freq1;
				lastminswrfreq2=freq2;
				return;
 			}
		}

		// check for relative encoder activity
		// note encoder res is halved to make it easier to set freq
		encoder = (long)POSCNT << 16;
		encoder = encoder >> 16;		// crude sign extention
		encoder /= 2;					// reduce control sensitivity
		if (encoder)
		{       timer_auto_off=AUTOOFFCNT;   // auto power-off
			POSCNT = 0;		// reset encoder
			if ( fieldnum == 0 )
			{
				freq1 += encoder * 250000L;
				if ( freq1 < eeb0.f_start ) freq1 = eeb0.f_stop;
				if ( freq1 > eeb0.f_stop ) freq1 = eeb0.f_start;
				//if ( freq1 > freq2 ) freq1 = freq2;
				int_freq1cal(freq1);
			}
			if ( fieldnum == 1 )
			{
				freq2 += encoder * 250000L;
				//if ( freq2 < freq1 ) freq2 = freq1;
				if ( freq2 > eeb0.f_stop ) freq2 = eeb0.f_start;
				if ( freq2 < eeb0.f_start ) freq2 = eeb0.f_stop;
				int_freq2(freq2);
			}
		}			
		// let outside world know we are alive (1hz flash)
		if ( timer_led == 0 )
		{
			//TODO read hardware to display voltages
			timer_led = 5000;			// 500ms to next update
			STATUS_LED = ~STATUS_LED;
		}
	}
}

//=============================================================================
// low level screen access to display resistance values in ohms,k or M
//=============================================================================
char* format_R( double Res )
{       if (fabs(Res) >= 1e6) 
	   sprintf(data,"%+07.2fM", Res/1e6);
	else if (fabs(Res) >= 1e3)
	   sprintf(data,"%+07.2fK", Res/1e3);
	else if (fabs(Res) >= 1)
	   sprintf(data,"%+07.2f ", Res);
	else
	   sprintf(data,"%+07.2fm", 1000*Res);
	return &data[0];
}

    
//=============================================================================
// low level screen access to display capacitance values in pF,nF,uF
//=============================================================================
char* format_C( double Cap )
{	if (fabs(Cap) >= 1)
	   sprintf(data,"%+07.2f ", Cap);
        else if (fabs(Cap) >= 1e-3) 
	   sprintf(data,"%+07.2fm", Cap*1e3);
	else if (fabs(Cap) >= 1e-6)
	   sprintf(data,"%+07.2fu", Cap*1e6);
	else if (fabs(Cap) >= 1e-9)
	   sprintf(data,"%+07.2fn", Cap*1e9);
	else
	   sprintf(data,"%+07.2fp", Cap*1e12);
	return &data[0];
}

//=============================================================================
// low level screen access to display inductance values in nH,uH,mH
//=============================================================================
char* format_L( double Ind )
{	if (fabs(Ind) >= 1)
	   sprintf(data,"%+07.2f ", Ind);
        else if (fabs(Ind) >= 1e-3) 
	   sprintf(data,"%+07.2fm", Ind*1e3);
	else if (fabs(Ind) >= 1e-6)
	   sprintf(data,"%+07.2fu", Ind*1e6);
	else if (fabs(Ind) >= 1e-9)
	   sprintf(data,"%+07.2fn", Ind*1e9);
	else
	   sprintf(data,"%+07.2fp", Ind*1e12);
	return &data[0];
}

//=============================================================================
// low level screen access to display 2nd line data on run screen
// 
//=============================================================================
void int_result( char result, long int freq, int compute )
{       //long int ReRdg, ImRdg;
	unsigned int bars;
	char dat[25] = {'S','W','R',0xff,0xff,0xff,
					0xff,0xff,0xff,0xff,0xff,0xff,
					0xff,0xff,0xff,0xff,0xff,0xff,
					0xff,0xff,0xff,0xff,0xff,0xff,0};
        char spaces[22] = {' ',' ',' ',' ',' ',
                           ' ',' ',' ',' ',' ',
                           ' ',' ',' ',' ',' ',
                           ' ',' ',' ',' ',' ',
                           ' ',0};
        #define RESULTDLY 5000
	// read hardware and convert to requested display
	if ( compute )
	{
           if (GetComplexRdg(freq, 1))  // br is used for 'break-in'
              return;
   	   CalcDUTparms(freq, Mu); 
	}
	else
	{
	   // this code is used when operator is changing display formats
	   lcd_goto(0x40);
	   lcd_puts("                        ");
	   lcd_goto(0x40);
	   switch (result)
	   {
	      case 6: lcd_puts("Zs@deg");  break;
	      case 9: lcd_puts("Rho");     break;
	      case 10:lcd_puts("Rho@deg"); break;
	      case 11:lcd_puts("Gp Bp");   break;
 	      case 1: lcd_puts("Rs Cs");   break;
	      case 2: lcd_puts("Rs Ls");   break;
	      case 3: lcd_puts("Rp Xp");   break;
	      case 4: lcd_puts("Rp Cp");   break;
	      case 5: lcd_puts("Rp Lp");   break;
	      case 8: lcd_puts("Q, RL");   break;
	      case 7: lcd_puts("SWR");     break;
	      case 0: lcd_puts("Rs Xs");   break;
	   }
	   //timer_led = 5000;
	   //while (timer_led);  //display the parm type for 500 ms
	   timerLEDdelay(RESULTDLY);  
	   return;
	}
	int_SWR( SWR );		// display swr
	//lcd_goto(0x40);
	//lcd_puts("                        ");
	lcd_goto(0x40);

	switch (result)
	{
	case 6:
                if (fabs(magZu) >= 1e6) 
	           sprintf(buff30,"Zs=%06.2fM @ %+07.2fDeg ",magZu/1e6,angZu);
	        else if (fabs(magZu) >= 1e3)
	           sprintf(buff30,"Zs=%06.2fK @ %+07.2fDeg ",magZu/1e3,angZu);
	        else if (fabs(magZu) >= 1)
	           sprintf(buff30,"Zs=%06.2f  @ %+07.2fDeg ",magZu,angZu);
	        else
	           sprintf(buff30,"Zs=%06.2fm @ %+07.2fDeg ",magZu*1e3,angZu);
		lcd_puts(buff30);
		break;

	case 9:
		//  Rho= +0000.00 +j0000.00
		if (Imrho < 0 )
			sprintf(buff30,"Rho=%+08.5f - j%07.5f",Rerho,-Imrho);
		else
			sprintf(buff30,"Rho=%+08.5f + j%07.5f",Rerho,Imrho);
		lcd_puts(buff30);
		break;

	case 10:
		sprintf(buff30,"Rho=%07.5f @ %+07.2fDeg ",magrho,angrho);
		lcd_puts(buff30);
		break;

	case 11:
		strcpy(buff30,"Gp=");
		strcat(buff30,format_C(Gp));
		strcat(buff30,"  Bp=");
		strcat(buff30,format_C(Bp));
		lcd_puts(buff30);
		break;

	case 1:
		strcpy(buff30,"Rs=");
		strcat(buff30,format_R(Rs));
		strcat(buff30,"  Cs=");
		strcat(buff30,format_C(Cs));
		lcd_puts(buff30);
		break;

	case 2:
		strcpy(buff30,"Rs=");
		strcat(buff30,format_R(Rs));
		strcat(buff30,"  Ls=");
		strcat(buff30,format_L(Ls));
		lcd_puts(buff30);
		break;

	case 3:
		strcpy(buff30,"Rp=");
		strcat(buff30,format_R(Rp));
		strcat(buff30,"  Xp=");
		strcat(buff30,format_R(Xp));
		lcd_puts(buff30);
		break;

	case 4:
		strcpy(buff30,"Rp=");
		strcat(buff30,format_R(Rp));
		strcat(buff30,"  Cp=");
		strcat(buff30,format_C(Cp));
		lcd_puts(buff30);
		break;

	case 5:
		strcpy(buff30,"Rp=");
		strcat(buff30,format_R(Rp));
		strcat(buff30,"  Lp=");
		strcat(buff30,format_L(Lp));
		lcd_puts(buff30);
		break;

	case 8:
               if (fabs(Q) >= 1e6) 
	           sprintf(buff30,"Q=%+07.2fM RL=%+08.3fdB",Q/1e6,RL);
	        else if (fabs(Q) >= 1e3)
	           sprintf(buff30,"Q=%+07.2fK RL=%+08.3fdB",Q/1e3,RL);
	        else if (fabs(Q) >= 1)
	           sprintf(buff30,"Q=%+07.2f  RL=%+08.3fdB",Q,RL);
	        else
	           sprintf(buff30,"Q=%+07.2fm RL=%+08.3fdB",Q*1e3,RL);
		lcd_puts(buff30);
		break;

	case 7:
		// SWR Bar graph
		{       bars = (unsigned int)((my_cabs(rho) * 21.0)+0.5);
			if ( bars > 21)
			   bars = 21;
			dat[bars+3] = '\0';
			strcpy(buff30,dat);
			spaces[21-bars] = '\0';
			strcat(buff30,spaces);
			lcd_puts(buff30);
			break;
		}

	case 0:
		//   Rs= +0000.00 Xs+0000.00
		strcpy(buff30,"Rs=");
		strcat(buff30,format_R(Rs));
		strcat(buff30,"  Xs=");
		strcat(buff30,format_R(Xs));
		lcd_puts(buff30);
		break;
	}
}

//=============================================================================
// low level screen access to display and manage the run screen
//	on this screen, the fieldnum (prev,next) buttons
//  determine which data is displayed on second line
//=============================================================================
void int_run( void )
{
	static int fieldnum = 0;		// local indicator which field gets encoder adj
	//static int resultnum = 0;		// 2nd line data display selection
	signed long freq1;
	long encoder;

	#define NUMFLD 9
	const int fieldpos[NUMFLD] = {5,7,8,9,10,11,12,13,0x40}; // screen posns for cursor

	// multipiers used for frequency selection
	const long int freqscale[] = {10000000L,1000000L,100000L,10000L,1000L,100L,10L,1L};

	if (lastreffreq)
	   freq1=lastreffreq;
	else
	   freq1 = eeb0.lastreffreq;					// last used ref freq 
        //resultrefnum initialized in in_main.c
        encoder=POSCNT=0;
	lcd_goto(0);
	lcd_puts("REF            SWR=     ");
	int_freq1(freq1);				// display real freq
	//AD9851_SetFreq( freq1, 0 );		// set VNA to new freq
        VNA_interpcalQ(freq1);
	int_result(resultrefnum,freq1, 0);	// real second line display
	lcd_goto(fieldpos[fieldnum]);	// set cursor properly

	while (1)
	{
		if ( but1 > 0 )
		{	timer_auto_off=AUTOOFFCNT;   // auto power-off
			fieldnum--;
			if ( fieldnum < 0 )
				fieldnum = NUMFLD-1;
			lcd_goto(fieldpos[fieldnum]);			// highlight freq variable
			while(but1 > 0);		// wait for button release
			timer_led = 1000;		// hold off readings untils buttons idle for 0.1sec
		}
		// next button
		if ( but2 > 0 )
		{       timer_auto_off=AUTOOFFCNT;   // auto power-off
			fieldnum++;
			if ( fieldnum >= NUMFLD )
				fieldnum = 0;
			lcd_goto(fieldpos[fieldnum]);			// highlight 2nd line data
			while(but2 > 0); 		//wait for button release
			timer_led = 1000;		// hold off readings until buttons idle 0.1sec
		}

   		// enter button	- exit to main menu
		if ( but3 > 0 )
		{       timer_auto_off=AUTOOFFCNT;   // auto power-off
			while( but3 > 0 );		// wait for button release
			break;
		}

		// check for relative encoder activity
		encoder = (long)POSCNT << 16;
		encoder = encoder >> 16;		// crude sign extention
		//encoder /= 4;					// shift not same as signed divide
		encoder /= 32;					// shift not same as signed divide
		if (encoder)
		{       timer_auto_off=AUTOOFFCNT;   // auto power-off
			// get here everytime encoder changes with magnitude of change in encoder
			POSCNT = 0;		// reset encoder
			if ( fieldnum == (NUMFLD-1) )
			{       
				// cursor is on second line... play with type of results displayed
				resultrefnum += encoder;
				if (resultrefnum < 0)		// wrap around in neg dir
					resultrefnum = 11;
				if (resultrefnum > 11)		// wrap around in pos posn
					resultrefnum = 0;
				int_result(resultrefnum,freq1,0);		// update second line
				lcd_goto(fieldpos[fieldnum]);
			}
			else
			{       
				freq1 += (signed long)(encoder * freqscale[fieldnum]);
				if (freq1 < (signed long)eeb0.f_start) 
				   freq1 = eeb0.f_stop;
				else if (freq1 > eeb0.f_stop) 
				   freq1 = eeb0.f_start;
				AD9851_SetFreq( freq1, 0 );			// set VNA to new freq
				int_freq1(freq1);
				VNA_interpcalQ(freq1);
				lcd_goto(fieldpos[fieldnum]);
				// readings are updated every 1/2 second below..
			}
			// to minimize lag, only update readings below
			// if the user has stopped twiddling encoder
			// setting timer will hold up calcs for 1 sec
			//if ((eeb0.samplerate ==2) || fieldnum == (NUMFLD-1))
			//	timer_led = 100;
			//else
			//	timer_led = 100;	// in fast/med mode, display updates asap
		}			
		// let outside world know we are alive (1hz flash)
		if ( timer_led == 0 )
		{
			STATUS_LED = ~STATUS_LED;
			//printf("Encoder=%ld POSCNT=%i\r\n",encoder, POSCNT);
                        int_result(resultrefnum,freq1,1);		// update second line
	                //lcd_goto(0);
	                //lcd_puts("REF            SWR=     ");
	                //int_freq1(freq1);				// display real freq
	                //int_SWR( SWR );		// display swr
			//if (resultrefnum==11)
			//   resultrefnum=0;
			lcd_goto(fieldpos[fieldnum]);
			if (eeb0.samplerate == 2)
				timer_led = 100;			// 10ms to next update
			else if (eeb0.samplerate == 1)
				timer_led = 200;			// 20ms to next update
			else
				timer_led = 500;			// 50ms to next update
			//timer_led = 100;       // 10ms til next update
		}
	}
	lastreffreq=freq1;
	eeb0.lastreffreq=lastreffreq;
	eeb0.lastresultrefnum=resultrefnum;
	writeCAL0();
}
 
char GetTrnMu(long freq, char br)
{  if (eeb0.Brdg == 2)      // Hi-Z
   {  Isensor();
      if (VNA_GetReadingCDS(freq, br ))
         return (1);
      __real__ Mu = ADCtoVolts(ReRdg);
      __imag__ Mu = ADCtoVolts(ImRdg);
   }
   else                       // RFIV, Lo-Z, or Std
   {  Vsensor();
      if (VNA_GetReadingCDS(freq, br ))
         return (1);
      __real__ Mu = ADCtoVolts(ReRdg);
      __imag__ Mu = ADCtoVolts(ImRdg);
   }
   Mu = Mu/Mt;   // assumes interpolation already done
   return (0);
}

char sign(double x)
{ if (x < 0)
     return -1;
  else
     return 1;
}

char grpdel(long int freq, long int delf, char br)
{    long fg1, fg2;
     double mg1, mg2, ag1, ag2, agd, mgm;
     fg1=freq-(delf/2);
     fg2=freq+(delf/2);  // fg2 > fg1
     if (fg1 < eeb0.f_start)
     {  fg1 = freq;
        fg2 = freq+(delf);
     }
     if (fg2 > eeb0.f_stop)
     {  fg2=freq;
        fg1=freq-(delf);
     }
     if (fg1 == fg2)    // narrow cal range - unlikely for delf max < 1MHz
        return (1);
     VNA_interpcalQ(fg1);
     if (GetTrnMu(fg1, 1))  // br is used for 'break-in'
        return (1);
     mg1=my_cabs(Mu);
     ag1=(180/M_PI)*my_arg(Mu);      // degrees
     VNA_interpcalQ(fg2);
     if (GetTrnMu(fg2, 1))  // br is used for 'break-in'
        return (1);
     mg2=my_cabs(Mu);
     ag2=(180/M_PI)*my_arg(Mu);      // degrees
     mgm=0.5*(mg1+mg2);
     gdb=20*log10(mgm+1e-38);     // mean gain in dB
     if ((sign(ag1) != sign(ag2)) && (180 < fabs(ag1-ag2)))
     {  if (sign(ag1)==-1)
           ag1 = 360 + ag1;
        else
           ag2 = 360 + ag2;
     }
     agd=ag1-ag2;
     //agm=0.5*(ag1+ag2);
     tg=agd/(36*(double)delf);             //units: deg, 0.1Hz, Sec
     //printf("fg1=%ld fg2=%ld gdb=%-.2f ag1=%-.3f ag2=%-.2f, agd=%-.2f,tg=%-.2fns\r\n",fg1,fg2,gdb,ag1,ag2,agd,tg);
     return (0);
}

void int_freq3(unsigned long freq)  // delta(f) display in group delay
{   lcd_goto(15);
    sprintf(buff30,"%06.2fkHz",(double)freq/1e4);
    lcd_puts(buff30);
    lcd_goto(20);
}
 
//=============================================================================
// low level screen access to display 2nd line data on run screen
// 
//=============================================================================
void int_resulttrn( char result, long int freq, int compute )
{       //long int ReRdg, ImRdg;
        //long delf;
	// read hardware and convert to requested display
	if ( compute )
	{  if (resulttrnnum != 2)
	   {  if (GetTrnMu(freq, 1))  // br is used for 'break-in'
                 return;
           }
           else
           {  if (grpdel(freq, delf, 1))  
                 return;
           }
	}
	else
	{
	   // this code is used when operator is changing display formats
	   lcd_goto(0x40);
	   lcd_puts("                        ");
	   lcd_goto(0x40);
	   switch (result)
	   {  case 2: lcd_puts("Gain: |G|@Group.Delay");   break;
	      case 1: lcd_puts("Gain: |G|@Deg");   break;
	      case 0: lcd_puts("Gain: dB@Deg");  break;
	   }
	   timerLEDdelay(5000);
           //timer_led = 5000;
	   //while (timer_led);  //display the parm type for 500 ms
	   return;
	}

	switch (result)
	{  case 2:
		int_freq3(delf);
		lcd_goto(0x40);                                                                                            //
		//sprintf(buff30,"G=%09.6f @ %+07.2fDeg",my_cabs(Mu),(double)((180.0/M_PI)*my_arg(Mu)));        
		//sprintf(buff30,"G=%07.2fdB @ %+07.2fns ",(double)(20.0*log10(my_cabs(Mu))),1e9*tg); 
		if (fabs(tg) >= 1e-3)
		   sprintf(buff30,"G=%07.2fdB @ %+07.2fms ",gdb,1e3*tg);  
		else if (fabs(tg) >= 1e-6)      
		   sprintf(buff30,"G=%07.2fdB @ %+07.2fus ",gdb,1e6*tg);  
		else if (fabs(tg) >= 1e-9)      
		   sprintf(buff30,"G=%07.2fdB @ %+07.2fns ",gdb,1e9*tg);
		else  
		   sprintf(buff30,"G=%07.2fdB @ %+07.2fps ",gdb,1e12*tg);
		lcd_puts(buff30);
		break;
	   case 1:
                lcd_goto(14);
                lcd_puts("          ");
                //lcd_goto(fieldpos[fieldnum]);
		lcd_goto(0x40);                                                                                            //
		sprintf(buff30,"G=%09.6f @ %+07.2fDeg",my_cabs(Mu),(double)((180.0/M_PI)*my_arg(Mu)));        
		lcd_puts(buff30);
		break;
	   case 0:
                lcd_goto(14);
                lcd_puts("          ");
                //lcd_goto(fieldpos[fieldnum]);
		lcd_goto(0x40);                                                                                            //
		sprintf(buff30,"G=%+07.2fdB @ %+07.2fDeg",(double)(20.0*log10(my_cabs(Mu))),(double)((180.0/M_PI)*my_arg(Mu)));        //case 2:
		lcd_puts(buff30);
		break;
	}
}

//=============================================================================
// low level screen access to display and manage the trn screen
//	on this screen, the fieldnum (prev,next) buttons
//  determine which data is displayed on second line
//=============================================================================
void int_trn( void )
{
	static int fieldnum = 0;		// local indicator which field gets encoder adj
	//static int resultnum = 0;		// 2nd line data display selection
	long freq1;
	long encoder;

	#define NUMFLDTRN 14
	const int fieldpos[NUMFLDTRN] = {5,7,8,9,10,11,12,13,15,16,17,19,20,0x40}; // screen posns for cursor

	// multipiers used for frequency selection
	const long int freqscale[] = {10000000L,1000000L,100000L,10000L,1000L,100L,10L,1L};
	const long int delfscale[] = {1000000L,100000L,10000L,1000L,100L};

	if (lasttrnfreq)
	   freq1=lasttrnfreq;
	else                            
	   freq1=eeb0.lasttrnfreq;    // initial freq is at low end of cal
	   
	if (lastdelf)
	   delf=lastdelf;
	else	
	   delf=eeb0.lastdelf;
	   
	//printf("eeb0.lastdelf=%ld sizeof(eeb0)=%i\r\n",eeb0.lastdelf,sizeof(eeb0));
	   
	if ((delf <= 100L) || (delf > 9999900L))
	   delf=10000L;  // 1 kHz
	   				
        //resulttrnnum initialized in int_main.c
        encoder=POSCNT=0;

	lcd_goto(0);
	lcd_puts("TRN                     ");
	int_freq1(freq1);				// display real freq
	//AD9851_SetFreq( freq1, 0 );		// set VNA to new freq
        VNA_interpcalQ(freq1);
	int_resulttrn(resulttrnnum,freq1, 0);	// real second line display
	lcd_goto(fieldpos[fieldnum]);	// set cursor properly
	if (resulttrnnum==2)
           int_freq3(delf);
        else
        {  lcd_goto(14);
           lcd_puts("          ");
           lcd_goto(fieldpos[fieldnum]);
        }
           
	while (1)
	{
		if ( but1 > 0 )
		{	timer_auto_off=AUTOOFFCNT;   // auto power-off
			fieldnum--;
			if ((resulttrnnum!=2) && (fieldnum>7) && (fieldnum<NUMFLDTRN-1))
			   fieldnum=7;
			if ( fieldnum < 0 )
			   fieldnum = NUMFLDTRN-1;
			lcd_goto(fieldpos[fieldnum]);	// highlight freq variable
			while(but1 > 0);		// wait for button release
			timer_led = 1000;		// hold off readings untils buttons idle for 0.1sec
		}
		// next button
		if ( but2 > 0 )
		{       timer_auto_off=AUTOOFFCNT;   // auto power-off
			fieldnum++;
			if ((resulttrnnum!=2) && (fieldnum>7) && (fieldnum<NUMFLDTRN-1))
			   fieldnum=NUMFLDTRN-1;
			if (fieldnum >= NUMFLDTRN)
			   fieldnum = 0;
			lcd_goto(fieldpos[fieldnum]);			// highlight 2nd line data
			while(but2 > 0); 		//wait for button release
			timer_led = 1000;		// hold off readings until buttons idle 0.1sec
		}
        
   		// enter button	- exit to main menu
		if ( but3 > 0 )
		{       timer_auto_off=AUTOOFFCNT;   // auto power-off
			while( but3 > 0 );		// wait for button release
			break;
		}
        
		// check for relative encoder activity
		encoder = (long)POSCNT << 16;
		encoder = encoder >> 16;		// crude sign extention
		//encoder /= 4;					// shift not same as signed divide
		encoder /= 32;					// shift not same as signed divide
		if (encoder)
		{       timer_auto_off=AUTOOFFCNT;   // auto power-off
			// get here everytime encoder changes with magnitude of change in encoder
			POSCNT = 0;		// reset encoder
			if (fieldnum == (NUMFLDTRN-1))
			{	// cursor is on second line... play with type of results displayed
				resulttrnnum += encoder;
				if (resulttrnnum < 0)		// don't wrap around in neg dir with only 2 options
					resulttrnnum = 0;
				if (resulttrnnum > 2)		// don't wrap around in pos posn
					resulttrnnum = 2;
				int_resulttrn(resulttrnnum,freq1,0);		// update second line
				lcd_goto(fieldpos[fieldnum]);
			}
			else if ((fieldnum>7) && (fieldnum<(NUMFLDTRN-1)))
			{	delf += encoder * delfscale[fieldnum-8];
		                if (delf < 100L) delf = 100L;
		                if (delf > 10000000L) delf = 9999900L;
		                int_freq3(delf);
				lcd_goto(fieldpos[fieldnum]);
		        }
			else
			{	freq1 += encoder * freqscale[fieldnum];
				if (freq1 < (signed long)eeb0.f_start) 
				   freq1 = eeb0.f_stop;
				else if (freq1 > eeb0.f_stop) 
				   freq1 = eeb0.f_start;
				AD9851_SetFreq(freq1, 0);			// set VNA to new freq
				int_freq1(freq1);
				//lcd_goto(14);
				//lcd_puts("          ");
				VNA_interpcalQ(freq1);   
				lcd_goto(fieldpos[fieldnum]);
				// readings are updated below..
			}
			// to minimize lag, only update readings below
			// if the user has stopped twiddling encoder
			// setting timer will hold up calcs for 1 sec
			//if ((eeb0.samplerate ==2) || fieldnum == (NUMFLD-1))
			//	timer_led = 100;
			//else
			//	timer_led = 100;	// in fast/med mode, display updates asap
		}	//		
		// let outside world know we are alive (1hz flash)
		if ( timer_led == 0 )
		{
			STATUS_LED = ~STATUS_LED;
			int_resulttrn(resulttrnnum,freq1,1);		// update second line
			lcd_goto(fieldpos[fieldnum]);
			if (eeb0.samplerate == 2)
				timer_led = 100;			// 10ms to next update
			else if (eeb0.samplerate == 1)
				timer_led = 200;			// 20ms to next update
			else
				timer_led = 500;			// 50ms to next update
		}
	}
	lasttrnfreq=freq1;
	lastdelf=delf;
	eeb0.lasttrnfreq=lasttrnfreq;
	eeb0.lastresulttrnnum=resulttrnnum;
	eeb0.lastdelf=lastdelf;
	writeCAL0();
	//printf("eeb0.lastdelf=%ld\r\n",eeb0.lastdelf);
}
