/* int_sdb.c  
 * dsPic30f4011 driver program for the N2PK Vector Network Analyzer
 * This file contains code for the operators interface.
 *
 *  Copyright (C) 2006 By Lawrence Glaister VE7IT
 *             (ve7it@shaw.ca)
 *
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Compile using: mplab ide
 * for some reason, this file pukes the compiler when using -O2,3 or s
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>		// for PI etc
#include "vna.h"

extern volatile int but1;				// -DEBOUNCE to +DEBOUNCE
extern volatile int but2;
extern volatile int but3;

extern volatile unsigned short int timer_test;
extern volatile unsigned short timer_led;
extern void AD9851_SetFreq( unsigned long int hz, unsigned long int phase );

// to conserve stack space define a general use char buffer as global
extern char buff30[30];

extern void lcd_goto( int );
extern void lcd_puts( char * );
extern char VNA_GetReadingCDS( long int freq, char br);  // br is used for 'break-in'
extern void int_cmplx( __complex__ num );
extern void CalcDUTparms(long int freq, __complex__ double Mu);
extern void VNA_interpcalQ( long int freq);
extern double ADCtoVolts(long int rdg);

extern unsigned int ADC10_read(void); 
extern void AD9851_Reset( void );

extern __complex__ double Mo,Ms,Ml,Mt,x,y,z;

extern double Rs,Xs,Cs,Ls;				
extern double Rp,Xp,Gp,Bp,Cp,Lp;			
extern double Q;					
extern double SWR,RL;					
extern double magrho,angrho,Rerho,Imrho;
extern double magZu,angZu,magYu,angYu;		
       
       // unknown reading
extern __complex__ double Mu;		        // raw voltages
extern __complex__ double rho;			// computed reflection coeff.
extern __complex__ double Zu,Yu;			// computed impedance
extern __complex__ double Zo;

struct EEbuffer0                         // 1st row (32 bytes)
{  long int refclk;			 // 156.25 MHz clk
   unsigned long int f_start, f_stop;	 // freq in 0.1 Hz for calibration points
   unsigned long int lastreffreq, lasttrnfreq;
   unsigned long int lastdelf;
   long int cksum;
   char samplerate;			 // a sample rate index: 0,1,2
   char Brdg;                             // 0=Std, 1=Lo-Z, 2=Hi-Z, 3=RFIV
   char lastresultrefnum,lastresulttrnnum;
};

extern char Brdg;    // 0=Std, 1=Lo-Z, 2=Hi-Z, 3=RFIV

extern __complex__ double Vrfiv,Irfiv;

extern struct EEbuffer0 eeb0;

extern void readCAL0(void);  // reads the first 32 bytes in EEPROM 
extern void writeCAL0(void);  // writes the first 32 bytes in EEPROM 

extern void Vsensor(void);
extern void Isensor(void);
extern long int ReRdg,ImRdg;
extern char GetComplexRdg(long int freq, char br);  // br is used for 'break-in'

//=============================================================================
// low level screen access to display and manage the serial debug screen
// This mode just takes continous readings and sends them out the serial port
// This was used to characterize drift issues on the VNA board
//=============================================================================
void int_sdb( void )
{
	unsigned long int freq;				// current freq 
	//long int fmax;
	unsigned long int serialcnt = 0;

	double battery;
	unsigned short ubat;
        //long int ReRdg=0,ImRdg=0;

	//fmax = cal.f_start + ((cal.f_num -1)*cal.f_step);
	// This test occurs at a fixed freq in the middle of the calibration range
	freq = ( eeb0.f_stop + eeb0.f_start ) / 2L;
 
	lcd_goto(0);
	lcd_puts("Sdb 12.245MHz I/O=12345 ");
	lcd_goto(0x40);
	lcd_puts("Zx=                     ");

	while (1)
	{
   		// enter button
		if ( but3 > 0 )
		{
			while( but3 > 0 );		// wait for button release
			break;
		}

		// let outside world know we are alive (1hz flash)
		// and update serial status displays
		if ( timer_led == 0 )
		{
			serialcnt++;
			// since we are trying to get reliable powerup syncronization
			// of the DDS chips, re reset them every 5 times to see if the
			// readings stay consistant. There is some tricky issue that after
			// a power up, we are not reading the same voltages from the bridge
			// making stored calibrations invalid. This test should show if
			// a reset of the DDS chips messes things up.
			if ( (serialcnt % 5) == 0 )
			{
				AD9851_Reset();
				printf("Reset DDS chips\r\n");
			}
			// read hardware and convert to requested display
			VNA_interpcalQ(freq);
 	                GetComplexRdg(freq, 0);
   	                CalcDUTparms(freq, Mu); 
			ubat = ADC10_read();
			battery = (double)ubat * BAT_SCALE;

			// send a line of numbers back to pc for analysis
			printf("%08lu %6.3f %08ld %8.6f %8.6f %8.5f %8.5f %8.3f %8.3f\r\n",
					serialcnt, battery, freq/10, __real__ Mu, __imag__ Mu, 
					Rerho, Imrho, Rs, Xs);

			timer_led = 10000;			// 1000ms to next update
			STATUS_LED = ~STATUS_LED;
			sprintf(buff30,"Sdb %08ldHz %08ld",freq/10,serialcnt);
			lcd_goto(0);
			lcd_puts(buff30);			// display current freq and cnt
			lcd_goto(0x43);				// display current impedance
			int_cmplx(Zu);
			lcd_goto(24);				// put cursor off screen
		}
	}
}
