/* int_tst.c  
 * dsPic30f4011 driver program for the N2PK Vector Network Analyzer
 * This file contains code for the operators interface.
 *
 *  Copyright (C) 2006 By Lawrence Glaister VE7IT
 *             (ve7it@shaw.ca)
 *
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Compile using: mplab ide
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>		// for PI etc
#include "vna.h"

extern volatile int but1;				// -DEBOUNCE to +DEBOUNCE
extern volatile int but2;
extern volatile int but3;

extern volatile unsigned short int timer_test;
extern volatile unsigned short timer_led;
extern void AD9851_SetFreq( unsigned long int hz, unsigned long int phase );

// to conserve stack space define a general use char buffer as global
extern char buff30[30];
extern void lcd_goto( int );
extern void lcd_puts( char * );
extern void int_freq1cal( unsigned long freq );
extern int VNA_GetReading( long int freq, __complex__ double *pResult );
extern char VNA_GetReadingCDS( long int freq, char br);  // br is used for 'break-in'
extern void int_lowv( double num );
extern void int_batt( void );
extern double ADCtoVolts(long int rdg);

extern double my_cabs( __complex__ double x );
extern double my_arg( __complex__ double x );
extern void VNA_interpcalQ( long int freq );

extern __complex__ double Mu;		        // raw voltages
extern __complex__ double Mt;		        // thru cal at freq

extern char Brdg;    // 0=Std, 1=Lo-Z, 2=Hi-Z, 3=RFIV

extern __complex__ double Vrfiv,Irfiv;

extern void Vsensor(void);
extern void Isensor(void);

extern long lasttstfreq;

struct EEbuffer0                         // 1st row (32 bytes)
{  long int refclk;			 // 156.25 MHz clk
   unsigned long int f_start, f_stop;	 // freq in 0.1 Hz for calibration points
   unsigned long int lastreffreq, lasttrnfreq;
   unsigned long int lastdelf;
   long int cksum;
   char samplerate;			 // a sample rate index: 0,1,2
   char Brdg;                             // 0=Std, 1=Lo-Z, 2=Hi-Z, 3=RFIV
   char lastresultrefnum,lastresulttrnnum;
};

extern struct EEbuffer0 eeb0;
extern long int ReRdg,ImRdg;
extern char GetComplexRdg(long int freq, char br);  // br is used for 'break-in'

//=============================================================================
// low level screen access to display and manage the test screen
// 
//=============================================================================
void int_tst( void )
{       //unsigned char EEbuffer[32];
	//long int ReRdg, ImRdg;
	long int freq=100000000L;	// start on wwv freq at entry to test mode
	long int encoder;

	lcd_goto(0);
	lcd_puts("TST                     ");
	lcd_goto(0x40);
	lcd_puts("                        ");
	int_batt();		// display battery voltage
	POSCNT = 0;		// reset encoder
	if (lasttstfreq)
	   freq=lasttstfreq;

	while (1)
	{
   		// enter button
		if ( but3 > 0 )
		{
			while( but3 > 0 );		// wait for button release
			break;
		}

		// check for relative encoder activity
		// note encoder res is halved to make it easier to set freq
		encoder = (long)POSCNT << 16;
		encoder = encoder >> 16;		// crude sign extention

		// 256 * 10khz / rev = 2.56mhz/rev
		if (encoder)
		{
			POSCNT = 0;		// reset encoder
			freq += encoder * 100000L;	// 10 kHz steps
			if ( freq < MINFREQ ) freq = MAXFREQ;
			if ( freq > MAXFREQ ) freq = MINFREQ;
			int_freq1cal(freq);
	    	        AD9851_SetFreq( freq, 0 );
			// hold off readings for 0.3 second
			timer_led = 3000;
		}			

		// let outside world know we are alive (1hz flash)
		// and update hardware status displays
		if ( timer_led == 0 )
		{
			timer_led = 5000;			// 500ms to next update
			STATUS_LED = ~STATUS_LED;

 	                GetComplexRdg(freq, 0);
			int_freq1cal(freq);			// display current frequency
			lcd_goto(0x40);
			sprintf(buff30,"R=%+07.2fdB @ %+07.2fDeg",(double)(20.0*log10(my_cabs(Mu))),(double)((180.0/M_PI)*my_arg(Mu)));
		        lcd_puts(buff30);
			//lcd_goto(24);				// put cursor off screen
			int_batt();
			lcd_goto(6);				// put cursor @ freq d.p.
		}
	}
	lasttstfreq=freq;
}


