/* int_utils.c  
 * dsPic30f4011 driver program for the N2PK Vector Network Analyzer
 * This file contains code for the operators interface.
 *
 *  Copyright (C) 2006 By Lawrence Glaister VE7IT
 *             (ve7it@shaw.ca)
 *
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Compile using: mplab ide
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>		// for PI etc
#include "vna.h"

extern volatile int but1;				// -DEBOUNCE to +DEBOUNCE
extern volatile int but2;
extern volatile int but3;

extern volatile unsigned short int timer_vna;
extern volatile unsigned short int timer_test;
extern volatile unsigned short timer_led;
extern void AD9851_SetFreq( unsigned long int hz, unsigned long int phase );

extern char buff30[];

extern void lcd_goto( int );
extern void lcd_puts( char * );
extern void int_cmplx( __complex__ num );
extern unsigned int ADC10_read(void);

void timerLEDdelay(int delay)
{    timer_led = delay;
     while (timer_led); 
} 

void timerVNAdelay(int delay)
{    timer_vna = delay;		// 1ms delay
     while (timer_vna);
}

//=============================================================================
// low level screen access to clear line 1 or 2
// 
//=============================================================================
void int_clr( int line )
{
	char *buff = {"                        "};

	if ( line == 1 )
		lcd_goto(0);
	else
		lcd_goto(0x40);
	lcd_puts(buff);
}

//=============================================================================
// low level screen access to display a frequency in position 1
// 
//=============================================================================
void int_freq1( unsigned long freq )
{
	ldiv_t result;
	result = ldiv(freq,10000000L);
	lcd_goto(4);
//	sprintf(buff30,"%6.3fMHz",(double)freq/1e6);
	sprintf(buff30,"%2lu.%07lu",result.quot,result.rem);
	lcd_puts(buff30);
	lcd_goto(6);			// so cursor is at d.p.
}

void int_freq1cal( unsigned long freq )
{
	ldiv_t result;
	result = ldiv(freq,10000000L);
	lcd_goto(4);
	sprintf(buff30,"%6.3fMHz",(double)freq/1e7);
	//sprintf(buff30,"%2lu.%07lu",result.quot,result.rem);
	lcd_puts(buff30);
	lcd_goto(6);			// so cursor is at d.p.
}

//=============================================================================
// low level screen access to display a frequency in position 2
// 
//=============================================================================
void int_freq2( unsigned long freq )
{
	lcd_goto(15);
	sprintf(buff30,"%6.3fMHz",(double)freq/1e7);
	lcd_puts(buff30);
	lcd_goto(17);			// so cursor is at d.p.

}

//=============================================================================
// low level screen access to display a complex number at current cursor posn
// 
//=============================================================================
void int_cmplx( __complex__ num )
{
	sprintf(buff30,"%8.2f %+8.2fi",__real__ num,__imag__ num);
	lcd_puts(buff30);
}
//=============================================================================
// low level screen access to display a floating number at current cursor posn
// like 1.00000 from the 24 bit adc's
//=============================================================================
void int_lowv( double num )
{
	sprintf(buff30,"%7.5f",num);
	lcd_puts(buff30);
}

//=============================================================================
// low level screen access to display the battery voltage
// 
//=============================================================================
void int_batt( void )
{
	double battery;
	unsigned short ubat;

	ubat = ADC10_read();
	battery = (double)ubat * BAT_SCALE;
	sprintf(buff30,"%6.2fv",battery+0.005);
	lcd_goto(17);
	lcd_puts(buff30);
}

//=============================================================================
// low level screen access to display the SWR
// 
//=============================================================================
void int_SWR( double swr )
{
	if (fabs(swr) >= 1000)
	   sprintf(buff30,"%s"," >1000");
	else if (fabs(swr) >=100)
	   sprintf(buff30,"=%5.1f",swr);
        else if (fabs(swr) >=10)
	   sprintf(buff30,"=%5.2f",swr);
	else
	   sprintf(buff30,"=%5.3f",swr);
	lcd_goto(18);
	lcd_puts(buff30);
}
